from __future__ import annotations

from tcsoa.gen.BusinessObjects import BusinessObject
from typing import List
from tcsoa.gen.Server import ServiceData
from tcsoa.utils import TcBaseObj
from dataclasses import dataclass
from datetime import datetime


@dataclass
class RbfNameValue(TcBaseObj):
    """
    The name of the input or output column along with the data value on a application extension rule.
    
    :var name: The input or output column name from the application extension rule.
    :var value: The value of the named column.
    """
    name: str = ''
    value: RbfValue = None


@dataclass
class RbfValue(TcBaseObj):
    """
    The value for an input or output column on an Application Extension Rule.
    
    :var dataType: The type of data the structure is holding. It will have one of the following values: ' STRING,
    BOOLEAN, INTEGER, DOUBLE, FLOAT, DATE, or TAG'.
    :var stringValue: The 'STRING' value for the column.
    :var booleanValue: The 'BOOLEAN' value for the column.
    :var integerValue: The 'INTEGER' value for the column.
    :var doubleValue: The 'DOUBLE' value for the column.
    :var floatValue: The 'FLOAT' value for the column.
    :var dateValue: The 'DATE' value for the column.
    :var tagValue: The 'TAG' value for the column.
    """
    dataType: str = ''
    stringValue: str = ''
    booleanValue: bool = False
    integerValue: int = 0
    doubleValue: float = 0.0
    floatValue: float = 0.0
    dateValue: datetime = None
    tagValue: BusinessObject = None


@dataclass
class ExecuteRbfRulesResponse(TcBaseObj):
    """
    Holds the response for the 'executeRbfRules' operation.
    
    :var outputs: This is a set of output name/value pairs from the rules engine execution.
    :var serviceData: This contains the status of the operation.
    """
    outputs: List[RbfNameValue] = ()
    serviceData: ServiceData = None



from __future__ import annotations

from tcsoa.gen.BusinessObjects import BusinessObject
from tcsoa.gen.Manufacturing._2010_09.TimeManagement import GetActivityTimesResponse, CalculateCriticalPathResponseEx
from tcsoa.gen.Manufacturing._2010_09.Core import FindNodeInContextResponse, GetAffectedPropertiesArg, FindNodeInContextInputInfo
from tcsoa.gen.Manufacturing._2010_09.ImportExport import ImportResponse, ImportInput
from typing import List
from tcsoa.gen.Server import ServiceData
from tcsoa.utils import TcService


class ImportExportService(TcService):

    @classmethod
    def importManufaturingFeatures(cls, input: ImportInput) -> ImportResponse:
        """
        imports MFGs from a given PLMXML file to TC.
        """
        return cls.execute_soa_method(
            method_name='importManufaturingFeatures',
            library='Manufacturing',
            service_date='2010_09',
            service_name='ImportExport',
            params={'input': input},
            response_cls=ImportResponse,
        )


class TimeManagementService(TcService):

    @classmethod
    def populateAllocatedTimeProperties(cls, rootNodes: List[BusinessObject], propagateZeroValues: bool, stopLevel: int, precedence: List[str]) -> ServiceData:
        """
        Updates the allocated time property recursively for all nodes of a list of process or operation trees.
        """
        return cls.execute_soa_method(
            method_name='populateAllocatedTimeProperties',
            library='Manufacturing',
            service_date='2010_09',
            service_name='TimeManagement',
            params={'rootNodes': rootNodes, 'propagateZeroValues': propagateZeroValues, 'stopLevel': stopLevel, 'precedence': precedence},
            response_cls=ServiceData,
        )

    @classmethod
    def updateTimeManagementProperties(cls, rootNodes: List[BusinessObject], fieldNames: List[str]) -> ServiceData:
        """
        Recomputes the cached values of the runtime properties related to TimeManagement for one or more process or
        operation trees. This will affect the properties of all nodes of the tree structures defined by the rootNodes
        parameter.
        """
        return cls.execute_soa_method(
            method_name='updateTimeManagementProperties',
            library='Manufacturing',
            service_date='2010_09',
            service_name='TimeManagement',
            params={'rootNodes': rootNodes, 'fieldNames': fieldNames},
            response_cls=ServiceData,
        )

    @classmethod
    def getActivityTimes(cls, rootNodes: List[BusinessObject]) -> GetActivityTimesResponse:
        """
        Traverses a set of process or operation trees and computes the effective accumulated times of all leaf
        activities, ignoring any flows between processes, operations or activities.  For each activity category that is
        encountered, a distinct value is returned.
        """
        return cls.execute_soa_method(
            method_name='getActivityTimes',
            library='Manufacturing',
            service_date='2010_09',
            service_name='TimeManagement',
            params={'rootNodes': rootNodes},
            response_cls=GetActivityTimesResponse,
        )

    @classmethod
    def calculateCriticalPathEx(cls, roots: List[BusinessObject], processLeafNodes: bool) -> CalculateCriticalPathResponseEx:
        """
        This operation computes the critical paths for processes or operations. The critical path is the sequence of
        processes or operations that determine the minimum duration of a specific object.  If the processLeafNodes flag
        is set, the algorithm will traverse down the structure until it finds all leaf nodes that make up the path.
        Otherwise only the direct children will be taken into account.
        """
        return cls.execute_soa_method(
            method_name='calculateCriticalPathEx',
            library='Manufacturing',
            service_date='2010_09',
            service_name='TimeManagement',
            params={'roots': roots, 'processLeafNodes': processLeafNodes},
            response_cls=CalculateCriticalPathResponseEx,
        )


class CoreService(TcService):

    @classmethod
    def findNodeInContext(cls, input: List[FindNodeInContextInputInfo]) -> FindNodeInContextResponse:
        """
        Finding parallel line in a given window of a given line.
        """
        return cls.execute_soa_method(
            method_name='findNodeInContext',
            library='Manufacturing',
            service_date='2010_09',
            service_name='Core',
            params={'input': input},
            response_cls=FindNodeInContextResponse,
        )

    @classmethod
    def getAffectedProperties(cls, arguments: List[GetAffectedPropertiesArg], requestedProperties: List[str]) -> ServiceData:
        """
        Returns the runtime properties of dependent nodes which are affected when the duration of one or more nodes has
        been changed in a process or operation structure.
        """
        return cls.execute_soa_method(
            method_name='getAffectedProperties',
            library='Manufacturing',
            service_date='2010_09',
            service_name='Core',
            params={'arguments': arguments, 'requestedProperties': requestedProperties},
            response_cls=ServiceData,
        )



from __future__ import annotations

from tcsoa.gen.Internal.ChangeManagement._2012_10.ChangeManagement import CreatableChangeTypesResponse, ContextDataInput
from typing import List
from tcsoa.utils import TcService


class ChangeManagementService(TcService):

    @classmethod
    def getCreatableChangeTypes(cls, inputs: List[ContextDataInput]) -> CreatableChangeTypesResponse:
        """
        This operation provides the list of ChangeItem types that are allowed to be created by the logged-in user.
        
        Use cases:
        Create Change in context
        User wants to create a Change (ProblemReport, ChangeRequest, ChangeNotice etc.) in context of one or more
        business objects. Based on context data and the Change creatable conditions defined in BMIDE allowed Change
        types will be returned.
        """
        return cls.execute_soa_method(
            method_name='getCreatableChangeTypes',
            library='Internal-ChangeManagement',
            service_date='2012_10',
            service_name='ChangeManagement',
            params={'inputs': inputs},
            response_cls=CreatableChangeTypesResponse,
        )



from __future__ import annotations

from tcsoa.gen.BusinessObjects import BusinessObject
from tcsoa.gen.ProjectManagementAw._2016_12.ScheduleManagementAw import EventDateRanges
from typing import List
from tcsoa.gen.Server import ServiceData
from tcsoa.utils import TcBaseObj
from dataclasses import dataclass


@dataclass
class ManageProgramViewResponse(TcBaseObj):
    """
    Response containing the ProgramView data comprising the new ProgramView created for 'create' operation, the list of
    ScheduleTask objects. The day and event ranges are returned for 'load' operation.
    
    :var programView: The context ProgramView object.
    :var configuration: The ProgramView data containing the list of Schedule UIDs, filters and groups. This is
    populated only if the input options in 'ProgramViewInfo' contain 'returnProgramViewConfig' entry.
    :var dayRanges: The list of Structure (int, list of string) containing the ranges for each day in a week. Maps the
    days of the week to their ranges. The valid values for days are 0 to 6 which represents Sunday through Saturday in
    the respective order. A range is a pair of start and end time of working hours mentioned in 24-hour format. A day
    can have multiple ranges with breaks in-between in a working day. This is populated only if the input options in
    'ProgramViewInfo' contain 'returnProgramViewConfig' entry.
    :var eventDateRanges: The event dates and their respective ranges. The events are exceptions where the working
    hours are different from the default working hours. This is populated only if the input options in
    'ProgramViewInfo' contain 'returnProgramViewConfig' entry.
    :var options: It represents any additional information to be returned in the response. Valid values are: 
    hasMore : true/false (Set to true if there are more objects to be loaded; false otherwise.)
    :var programViewNodes: The list of ProgramView nodes loaded.
    :var programViewNodesJson: The list of ProgramView nodes loaded in JSON format.
    :var serviceData: The service data containing the ProgramView object in the created or updated list based on the
    operationType and ScheduleTask objects in the plain objects list.
    """
    programView: BusinessObject = None
    configuration: ProgramViewConfiguration = None
    dayRanges: List[DayRanges3] = ()
    eventDateRanges: EventDateRanges = None
    options: List[NameValueStringPair] = ()
    programViewNodes: List[ProgramViewNode] = ()
    programViewNodesJson: str = ''
    serviceData: ServiceData = None


@dataclass
class NameValueStringPair(TcBaseObj):
    """
    Structure containing a pair(string,string) of name and value respectively.
    
    :var name: The name of key in Pair.
    :var stringValue: The value of key in Pair.
    """
    name: str = ''
    stringValue: str = ''


@dataclass
class ProgramViewConfiguration(TcBaseObj):
    """
    Contains the configuration data used for managing a ProgramView.
    
    :var scheduleUIDs: List of Schedule UIDs of the ProgramView.
    :var columns: List of ProgramView columns. The columns specify the ProgramView properties to be loaded.
    :var groups: List of ProgramView groups. The ProgramView nodes are grouped based on the properties specified in
    each group.
    :var filterSets: List of ProgramView filter set. Each FilterSet is an array of filters to be applied to the
    ProgarmView. The results of each FilterSet are combined together to build the final ProgramView structure.
    """
    scheduleUIDs: List[str] = ()
    columns: List[Column] = ()
    groups: List[ProgramViewGroup] = ()
    filterSets: List[FilterSet] = ()


@dataclass
class ProgramViewGroup(TcBaseObj):
    """
    Contains information about the Group modifier to be applied to the ProgramView.
    
    :var attributeName: The name of the grouping attribute in a ProgramView. This should be in the format of
    'Type.property'. For example, ScheduleTask.start_date.
    :var range: The range to be used for grouping the values.
    :var color: The color to be used for the group node.
    :var order: The order of the groups in a ProgramView. Valid values are: "none", "ascending", "descending"
    :var rollups: List of rollups for a group with rollup name as the key and value as the option. The rollup name
    should be in the format of 'Type.property'. For example, ScheduleTask.start_date. Valid values for rollup option
    are 'minimum', 'maximum', 'count', 'median', 'sum', 'average'.
    """
    attributeName: str = ''
    range: str = ''
    color: str = ''
    order: str = ''
    rollups: List[NameValueStringPair] = ()


@dataclass
class ProgramViewInfo(TcBaseObj):
    """
    Contains information about the operation to perform on the ProgramView and the configuration to use. The context
    node and reference node is provided in case of load operation
    
    :var programView: The ProgramView object in context. This will be NULL for &rsquo;create&rsquo; operation and a new
    ProgramView will be created.
    :var operationType: The operation to perform on the ProgramView. Valid values are:
    create : Creates a new ProgramView using the input programViewConfiguration.
    load : Loads the ProgramView
    loadUsingInputConfig : Loads the ProgramView using the input programViewConfiguration.
    save : Saves the ProgramView using the input programViewConfiguration.
    :var contextNodeId:  The unique identifier of the parent context node to load the child nodes. This input is
    applicable only for 'load' and &rsquo; loadUsingInputConfig&rsquo; operation types.
    :var referenceNodeId: The unique identifier of the reference node to be used for pagination. This input is
    applicable only for 'load' and &lsquo; loadUsingInputConfig&rsquo; operation types.
    :var inputOptions:  A list of options used for the ProgramView operation.
    Valid options and values are:
    returnProgramViewConfig : true/false ( Set true to return the current ProgramView configuration; false otherwise )
    returnResponseAsJSON : true/false ( Set true to return the ProgramView nodes in JSON format; false otherwise )
    :var programViewConfiguration: The ProgramView configuration containing Schedule UIDs, filters and group modifiers.
    This input is applicable only for ' loadUsingInputConfig&rsquo; and 'save' operation types.
    """
    programView: BusinessObject = None
    operationType: str = ''
    contextNodeId: str = ''
    referenceNodeId: str = ''
    inputOptions: List[NameValueStringPair] = ()
    programViewConfiguration: ProgramViewConfiguration = None


@dataclass
class ProgramViewNode(TcBaseObj):
    """
    Contains information about a ProgramView node.
    
    :var isTcObject: Set to true if the node represents a Teamcenter business object; false otherwise.
    :var nodeProperties: A list of Structure(string,string) of property names and its values for a ProgramView node.
    """
    isTcObject: bool = False
    nodeProperties: List[NameValueStringPair] = ()


@dataclass
class Column(TcBaseObj):
    """
    It specifies a column and it&rsquo;s sort order in a ProgramView.
    
    :var columnName: The name of the column in a ProgramView. This should be in the format of 'Type.property'. For
    example, ScheduleTask.start_date.
    :var order: The sorting order of the column. Valid values are: "none", "ascending", "descending".
    """
    columnName: str = ''
    order: str = ''


@dataclass
class DayRanges3(TcBaseObj):
    """
    Maps the weekdays to their ranges. The valid values for the days are 0 to 6 which represents Sunday through
    Saturday in the respective order. A range is a pair of start and end time of working hours. A day can have multiple
    ranges with breaks in-between in a working day.
    
    :var day: The event day.
    :var ranges: An array of ranges for the event day.
    """
    day: int = 0
    ranges: List[str] = ()


@dataclass
class Filter(TcBaseObj):
    """
    Contains information about a ProgramView filter.
    
    :var attributeName: The name of the filter. This should be in the format of 'Type.property'. For example,
    ScheduleTask.start_date.
    :var criteria: The filter criteria. Valid values are: 'equal','notEqual','greaterThan',
    'lessThan','greaterThanOrEqualTo','lessThanOrEqualTo', &lsquo;between'
    :var filterValue: The value of the ProgramView object property used for filtering.
    """
    attributeName: str = ''
    criteria: str = ''
    filterValue: str = ''


@dataclass
class FilterSet(TcBaseObj):
    """
    Contains a list of ProgramView filters. A FilterSet is an array of filters to be applied to the ProgramView.
    
    :var filters: List of ProgramView filters.
    """
    filters: List[Filter] = ()



from __future__ import annotations

from tcsoa.gen.Internal.Query._2012_02.SavedQuery import DescribeSavedQueryDefinitionInput
from tcsoa.gen.Internal.Query._2013_05.SavedQuery import DescribeSavedQueryDefinitionsResponse2
from typing import List
from tcsoa.utils import TcService


class SavedQueryService(TcService):

    @classmethod
    def describeSavedQueryDefinitions2(cls, requestedQueries: List[DescribeSavedQueryDefinitionInput]) -> DescribeSavedQueryDefinitionsResponse2:
        """
        This operation returns detailed information about a saved query, including the search criteria and default sort
        attributes.  The search criteria is made up of a list of attributes and the parameters used to evaluate them. 
        The end user running the query will be able to enter a search value for each attribute listed in the search
        criteria.  The criteria will indicate the math operation ("!=", ">", etc) to use when evaluating the users
        input.  Default values are also included.
        The sort attributes are a way to define a default sorting to use whenever the saved query is executed.  For
        each attribute, an "Ascending" or "Descending" sort order will be included.  Any user running the saved query
        will see the results sorted according to the sorting defined on the saved query.  The end user can use the Sort
        dialog to override what is defined.
        All of the attributes in both lists will include the L10N key to be used when displaying the attribute to the
        user.  This is part of the Query Builder saved query definition.
        """
        return cls.execute_soa_method(
            method_name='describeSavedQueryDefinitions2',
            library='Internal-Query',
            service_date='2013_05',
            service_name='SavedQuery',
            params={'requestedQueries': requestedQueries},
            response_cls=DescribeSavedQueryDefinitionsResponse2,
        )



from __future__ import annotations

from tcsoa.gen.BusinessObjects import POM_object, Fnd0EditContext
from typing import List
from tcsoa.gen.Server import ServiceData
from tcsoa.utils import TcService


class DataManagementService(TcService):

    @classmethod
    def setEditContext(cls, context: Fnd0EditContext, objects: List[POM_object]) -> ServiceData:
        """
        If input parameter "context" is not null, the SOA response will contain the input Business Objects configured
        with the configuration parameter belonging to the input context. If any of the input Business Objects are not
        POM Revisable, they will be returned without configuration parameter. The Business Objects will be added to the
        Plain Objects array in ServiceData.
        If input parameter "context" is null, the input Business Objects will be configured for Public edits and
        returned in the SOA response.
        
        Use cases:
        This operation will enable the user to enter or exit edit mode and subsequently save the edits as Private Edits.
        """
        return cls.execute_soa_method(
            method_name='setEditContext',
            library='EditContext',
            service_date='2014_12',
            service_name='DataManagement',
            params={'context': context, 'objects': objects},
            response_cls=ServiceData,
        )



from __future__ import annotations

from typing import List
from tcsoa.gen.BusinessObjects import Schedule
from tcsoa.utils import TcBaseObj
from dataclasses import dataclass


@dataclass
class MasterScheduleCreateInput(TcBaseObj):
    """
    Input structure containing information to create the master Schedule.
    
    :var name: Name of the Master Schedule.
    :var description: Descrption of the master Schedule.
    :var schedulesToInsert: A list of Schedule to be inserted into the master Schedule.
    :var timeZone: Specifies Time Zone for the master Schedule.
    """
    name: str = ''
    description: str = ''
    schedulesToInsert: List[Schedule] = ()
    timeZone: str = ''



from __future__ import annotations

from tcsoa.gen.BusinessObjects import ItemRevision, BOMLine
from tcsoa.gen.StructureManagement._2014_10.Structure import CloneStructureInputInfo, CloneStructureExpandOrUpdateResponse, CloneStructureResponse, CloneStructureExpandOrUpdateItemsInfo
from tcsoa.gen.StructureManagement._2014_10.MassUpdate import ExecuteMarkupChangeResponse, ImpactedObjectsQueryInput, GetMarkupChangesForUpdateResponse, ImpactedObjectDetailsInput, ImpactedObjectDetailsResponse, UpdateImpactedObjectEndResponse, GetRevisionRulesResponse, ManageImpactedObjectUpdateInput, ManageImpactedObjectUpdatesResponse, UpdateImpactedObjectInput, ExecuteMarkupChangeInput, UpdateImpactedObjectStartResponse, ImpactedObjectsQueryResponse, ExpandOneLevelSearchScopeInput, ValidateChangeObjectForMassUpdateResponse, ExpandOneLevelSearchScopeResponse, UpdateImpactedObjectResponse
from typing import List
from tcsoa.gen.Server import ServiceData
from tcsoa.utils import TcService


class MassUpdateService(TcService):

    @classmethod
    def getRevisionRules(cls, operation: int) -> GetRevisionRulesResponse:
        """
        This operation returns the list of valid revision rules applicable for specific mass update operation type.
        """
        return cls.execute_soa_method(
            method_name='getRevisionRules',
            library='StructureManagement',
            service_date='2014_10',
            service_name='MassUpdate',
            params={'operation': operation},
            response_cls=GetRevisionRulesResponse,
        )

    @classmethod
    def manageImpactedObjectUpdates(cls, input: List[ManageImpactedObjectUpdateInput]) -> ManageImpactedObjectUpdatesResponse:
        """
        This operation performs Add impacted object to or Remove impacted object from Change ItemRevision Markup as per
        the client request
        """
        return cls.execute_soa_method(
            method_name='manageImpactedObjectUpdates',
            library='StructureManagement',
            service_date='2014_10',
            service_name='MassUpdate',
            params={'input': input},
            response_cls=ManageImpactedObjectUpdatesResponse,
        )

    @classmethod
    def updateImpactedObjects(cls, input: List[UpdateImpactedObjectInput], updateRequestId: str) -> UpdateImpactedObjectResponse:
        """
        This operation performs an update on input impacted objects by replacing target ItemRevision with replacement
        ItemRevision. 
        """
        return cls.execute_soa_method(
            method_name='updateImpactedObjects',
            library='StructureManagement',
            service_date='2014_10',
            service_name='MassUpdate',
            params={'input': input, 'updateRequestId': updateRequestId},
            response_cls=UpdateImpactedObjectResponse,
        )

    @classmethod
    def updateImpactedObjectsEnd(cls, input: List[UpdateImpactedObjectInput]) -> UpdateImpactedObjectEndResponse:
        """
        This operation performs the post update cleanup activity. This operation should be called only once after all
        the update batches are executed using 'updateImpactedObjects'. If all the update batches are executed
        successfully then this operation deletes the markup associated with input change ItemRevision. 
        
        Use cases:
        - User wants to update the realizations saved on change ItemRevision. Operation 'getMarkupChangesForUpdate' is
        called by providing change ItemRevision as an input. This operation returns the Fnd0MarkupChange objects having
        information about impacted objects to be updated.
        - If the number of markup change objects is more than the update batch size then objects are updated in
        batches. Operation 'executeMarkupChanges' is then called per batch to update the batched markup change objects.
        This operation performs update on the impacted objects using information available on Fnd0MarkupChange objects.
        - Operation 'updateImpactedObjectsEnd' ends the update process and performs the cleanup activity if required.
        E.g. If the update is performed in context of change ItemRevision and if all the batches are executed
        successfully then this operation deletes the Fnd0Markup object associated with change ItemRevision in context
        of which the update has been performed. 
        
        """
        return cls.execute_soa_method(
            method_name='updateImpactedObjectsEnd',
            library='StructureManagement',
            service_date='2014_10',
            service_name='MassUpdate',
            params={'input': input},
            response_cls=UpdateImpactedObjectEndResponse,
        )

    @classmethod
    def updateImpactedObjectsStart(cls, input: List[UpdateImpactedObjectInput]) -> UpdateImpactedObjectStartResponse:
        """
        This operation builds the prerequisite information required to perform an update in batches. E.g. this
        operation generates an update request identifier that is used by all update batch requests. This operation
        should be called only once before calling the 'updateImpactedObjects' operation per batch.
        
        Use cases:
        - User wants to update the realizations (of a target ItemRevision) to a specific replacement ItemRevision. To
        do the update all impacted objects needs to be identified. Operation 'getImpactedObjects' is called by
        providing an input target ItemRevision as an input. This operation returns all the impacted objects.
        - Operation 'getImpactedObjectDetails' is then called by providing all the impacted objects as an input to
        check whether the impacted objects are modifiable or not.
        - User then selects the modifiable impacted objects to perform update upon them. If the number of selected
        impacted objects is more than the update batch size then impacted objects are updated in batches. Operation
        'updateImpactedObjectsStart' starts the update process with some preprocessing. e.g. operation
        'updateImpactedObjectsStart' builds the update request identifier that is common for all the subsequent update
        batch requests. Operation 'updateImpactedObjectsStart' should be called only once before calling the
        'updateImpactedObjects' operation per batch
        - Operation 'updateImpactedObjects' is then called per batch to update the batched impacted objects. This
        operation updates the impacted objects as per the input replacement ItemRevision.
        
        """
        return cls.execute_soa_method(
            method_name='updateImpactedObjectsStart',
            library='StructureManagement',
            service_date='2014_10',
            service_name='MassUpdate',
            params={'input': input},
            response_cls=UpdateImpactedObjectStartResponse,
        )

    @classmethod
    def validateChangeObjectForMassUpdate(cls, input: List[ItemRevision], massUpdateType: str) -> ValidateChangeObjectForMassUpdateResponse:
        """
        This operation validates the input change ChangeItemRevision object for a given Mass Update type. When invoked
        with "massUpdate" as an input, it validates whether the input ChangeItemRevision is valid for 'Mass Update'
        action. When invoked with "massUpdateRealization" as an input, it validates whether the input
        ChangeItemRevision is valid for 'Mass Update Realization' action.
        
        Use cases:
        Select ChangeItemRevision and perform 'Mass Update' or 'Mass Update Realization'. Operation
        'validateChangeObjectForMasUpdate' is invoked to validate whether the selected ChangeItemRevision is valid for
        the respective action 'Mass Update' or 'Mass Update Realization' initiated by the user.
        """
        return cls.execute_soa_method(
            method_name='validateChangeObjectForMassUpdate',
            library='StructureManagement',
            service_date='2014_10',
            service_name='MassUpdate',
            params={'input': input, 'massUpdateType': massUpdateType},
            response_cls=ValidateChangeObjectForMassUpdateResponse,
        )

    @classmethod
    def executeMarkupChanges(cls, input: List[ExecuteMarkupChangeInput]) -> ExecuteMarkupChangeResponse:
        """
        This operation executes the markup change objects corresponding to input Fnd0MarkupChange object UID list. Each
        markup change object holds the information like object to be modified, the input required for object
        modification and type of modification. When markup change object is executed, it modifies the required object
        with the help of information available on markup change object.
        
        Use cases:
        Execute markup changes associated with ChangeItemRevision
        - Invoke MassUpdate service operation getImpactedObjects by providing target ItemRevision. This API returns the
        list of impacted object UIDs.
        - Invoke MassUpdate service operation getImpactedObjectDetails by supplying the batch of impacted object UIDs.
        This API returns the information specifying whether each of the input impacted objects is modifiable or not.
        - Invoke MassUpdate service operation manageImpactedObjectUpdates providing the list of modifiable impacted
        objects and a ChangeItemRevision. This service operation when invoked with
        executionMode=murAddImpactedObjectToMarkup, adds the input impacted objects on the input ChangeItemRevision.
        - Invoke MassUpdate service operation getMarkupChangesForUpdate by providing a ChangeItemRevision as input. It
        returns the list of Fnd0MarkupChange object UIDs referenced by Fnd0Markup object associated with input
        ChangeItemRevision. 
        - Invoke MassUpdate service operation  executeMarkupChanges in loop by providing a batch of Fnd0MarkupChange
        objects UIDs until all the Fnd0MarkupChange UIDs are processed for execution. Once all the batches are executed
        successfully then delete the Fnd0Markup object associated with input ChangeItemRevision.
        
        """
        return cls.execute_soa_method(
            method_name='executeMarkupChanges',
            library='StructureManagement',
            service_date='2014_10',
            service_name='MassUpdate',
            params={'input': input},
            response_cls=ExecuteMarkupChangeResponse,
        )

    @classmethod
    def expandOneLevel(cls, input: List[ExpandOneLevelSearchScopeInput]) -> ExpandOneLevelSearchScopeResponse:
        """
        This operation expands the given parent product structure node to fetch its immediate children. A call to this
        operation is made when defining a search scope for impacted object search.
        """
        return cls.execute_soa_method(
            method_name='expandOneLevel',
            library='StructureManagement',
            service_date='2014_10',
            service_name='MassUpdate',
            params={'input': input},
            response_cls=ExpandOneLevelSearchScopeResponse,
        )

    @classmethod
    def getImpactedObjectDetails(cls, input: List[ImpactedObjectDetailsInput]) -> ImpactedObjectDetailsResponse:
        """
        This operation gathers the impacted object details required to perform mass update. Details like whether the
        impacted object is selectable for performing an update, if not selectable then the reason for it being
        non-selectable, whether the impacted object is out of date etc. is returned as an output response.
        """
        return cls.execute_soa_method(
            method_name='getImpactedObjectDetails',
            library='StructureManagement',
            service_date='2014_10',
            service_name='MassUpdate',
            params={'input': input},
            response_cls=ImpactedObjectDetailsResponse,
        )

    @classmethod
    def getImpactedObjects(cls, input: List[ImpactedObjectsQueryInput]) -> ImpactedObjectsQueryResponse:
        """
        This operation searches for product structure objects where  a given target ItemRevision is used.
        """
        return cls.execute_soa_method(
            method_name='getImpactedObjects',
            library='StructureManagement',
            service_date='2014_10',
            service_name='MassUpdate',
            params={'input': input},
            response_cls=ImpactedObjectsQueryResponse,
        )

    @classmethod
    def getMarkupChangesForUpdate(cls, input: List[ItemRevision]) -> GetMarkupChangesForUpdateResponse:
        """
        This operation queries Fnd0MarkupChange objects referenced by Fnd0Markup object associated with input
        ChangeItemRevision and returns UIDs of the Fnd0MarkupChange objects to the client.
        
        Use cases:
        Execute markup changes associated with ChangeItemRevision object
        - Invoke MassUpdate service operation getImpactedObjects by providing target ItemRevision. This operation
        returns the list of impacted object UIDs.
        - Invoke MassUpdate service operaion getImpactedObjectDetails by supplying the batch of impacted object UIDs.
        This API returns the information specifying whether each of the input impacted objects is modifiable or not.
        - Invoke MassUpdate service operation manageImpactedObjectUpdates providing the list of modifiable impacted
        objects and a ChangeItemRevision. This API when invoked with executionMode=murAddImpactedObjectToMarkup, adds
        the input impacted objects on the input ChangeItemRevision.
        - Invoke MassUpdate service operation getMarkupChangesForUpdate by providing a ChangeItemRevision as input. It
        returns the list of Fnd0MarkupChange object UIDs referenced by Fnd0Markup object associated with input
        ChangeItemRevision. 
        - Invoke MassUpdate service operation executeMarkupChanges in loop by providing a batch of Fnd0MarkupChange
        object UIDs until all the Fnd0MarkupChange UIDs are processed for execution.
        
        """
        return cls.execute_soa_method(
            method_name='getMarkupChangesForUpdate',
            library='StructureManagement',
            service_date='2014_10',
            service_name='MassUpdate',
            params={'input': input},
            response_cls=GetMarkupChangesForUpdateResponse,
        )


class StructureService(TcService):

    @classmethod
    def cloneStructure(cls, inputs: List[CloneStructureInputInfo]) -> CloneStructureResponse:
        """
        This operation validates and creates a duplicate (clone) of the input structure from its top level down or a
        selected sub assembly.
        
        Depending on the input arguments, all or some of the original structure is duplicated and the rest are
        referenced, revised or replaced.
        
        The caller can define a specific naming pattern for the Item ids of the duplicated (cloned) structure. The
        caller can define specific Item ids for individually selected ItemRevision objects or a default naming pattern
        for the duplicated structure. The default pattern can be defined by adding prefixes, suffixes or replacing part
        of the original name with a different pattern. The caller can also choose to allow the system to assign default
        ids.
        
        If project(s) have been passed in as input, the cloned structure is assigned to that project(s). By default the
        projects to which the top BOMLine in the duplicate dialog belongs and to which the user has access, is used to
        populate the project list. The user has the option to modify that list based on which projects are available to
        that user.
        
        All of the structure dependent data of the input structure like Datasets and attachments are copied to the
        duplicated structure based on the Business Modeler IDE Deep Copy Rules for SaveAs or the Deep Copy Data
        override rules parameter passed in to the input structure. The duplicate operation internally uses SaveAs at
        every level of the structure; therefore it uses the SaveAs Deep Copy Rules.
        
        If the structure being cloned is a Requirements Manager structure,Tracelink GRMs are handled based on the deep
        copy rules set for ReqRev for SaveAs.
        CAD specific attachments and relations are copied based on the transfer mode defined for the Business Modeler
        IDE global constant StructureCloneTransferModes. The transfer mode contains dependent closure rules for
        expansion and cloning. The value for the closure rules is added by the installed CAD system. 
        
        The caller can also tell the operation to just validate only and do not perform a duplicate of the input
        structures.
        
        Note: The difference between the operations duplicate4 and cloneStructure are the following:
        
        Duplicate4 and cloneStructure
        - cloneStructure was created as a result of the project to get NX CAD on board. The difference is
        cloneStructure will now process set based inputs, it combines the validate and duplicate actions into one API,
        provides a validate only mode, introduces the ability to override DeepCopyData GRM rules to change the core
        default DeepCopy rules behavior (with exception of restricted rules), the ability to choose the folder to store
        new cloned root item revisions into, added two new action types revise and replace along with the reference and
        clone action types for determining what to do with the children of the structure being cloned and  the
        cloneStructure will return all cloned mapping information to client if requested by cloneFlags.
        
        
        Use cases:
        Use case1: Simple Clone
        
        A user has an assembly which does not have cad dependencies nor does it belong to a specific project(s). The
        user wants to duplicate the entire structure with a specific naming pattern for the ItemIds. The naming pattern
        is a prefix "test_".
        The user invokes the duplicate operation by passing in the top BOMLine of the structure to be cloned, and the
        naming pattern for the new structure. The result is a new structure with the following naming pattern for the
        ItemIds -> test_OriginalItemId.
        
        Use case2: CAD Clone
        
        A user has an assembly structure which has cad dependencies. The user wants to start a new product with a
        similar structure and cad dependencies. The expansion and cloning rules have been defined in the Business
        Modeler IDE global constant StructureCloneTransferModes. 
        The user invokes the duplicate operation by passing in the top BOMLine of the structure to be cloned. 
        The user selects the cad dependency option Part Family Master. The expansion and cloning will be done based on
        the closure rules defined for Part Family Master in the CAD closure rules.
        The "Rename Cad Files" will be passed to the CAD integration in a callback. If the "Rename Cad Files", is set
        to true by the user, the cad files need to be renamed by the cad integration.
        The result will be a duplicated structure with the expected cad dependencies and it will open in the CAD tool
        without any errors.
        
        Use case3: Requirements Manager (Systems Engineering) clone:
        
        The user has a requirements manager structure with internal and/or external tracelink GRMs that need to be
        copied to the cloned structure. The user defines a set of projects to which the newly cloned structure should
        belong. The user does not want to clone the entire structure only a sub-assembly.
        The precondition to this operation, is that the deep copy rules for SaveAs have been setup correctly
        The user invokes the duplicate operation by passing in the selected BOMLine of the sub structure to be cloned.
        The projects to which the cloned structure should belong are passed in as input. The naming rule for the ItemId
        is passed in. 
        The result is a requirement manager structure with the tracelink relations pointing to the correct objects in
        the new structure. And the newly cloned structure belongs to the defined projects for which the user has
        permissions.
        
        Use Case 4: Validate and Duplicate Process with Revise and Replace Child Component Options
        
        A user has an assembly to start a new product with a similar structure but wants to replace and revise some
        components for the new structure. The user will need to select the components to revise from the client
        interface and mark those selected as a revise action to be recorded in the data map. Then the user will need to
        select the components to replace from the client interface and enter an existing component to be the replacing
        component for the components selected for replace. The replacing action will also be recorded in the data map.
        The results will be a cloned structure where the children selected for replace and revise will be replaced and
        revised in the new cloned structure.
        
        Use Case 5: Validate Only Mode
        
        A user has an assembly and wants to validate it before trying to clone it. User selects the assembly structure
        and selects the "Validate Only"  option.
        The results is the structure will run through a validation routine and not be cloned at all. The information of
        the validation will be returned to the client where the user then can fix any issues or proceed with the
        cloning.
        
        Use Case 6: Run In Background Mode
        
        A user has a very large assembly and wants to clone it in background mode so they can free up there client
        interface to perform other work. The user selects the assembly structure and selects the "Run In Background" 
        option.
        The system will return a message saying the job was dispatched. Then the system will validate the structure
        against the input provided and then duplicate the structure in an Asynchronous Teamcenter server thread. The
        results of the "Run In Background" process will be recorded in a text dataset that will be sent to the user via
        Teamcenter Mail Envelope.
        
        Use Case 7: Simple Validate and Duplicate Process And Return Cloned Data to Client
        
        A user has an assembly and wants to the cloned information to be returned to the client. The user selects the
        assembly structure and sets the "return cloned object information"  option for the cloneFlags.
        The results of the "return cloned object information " option will be returned to the client.
        Note: The "return cloned object information " option is primarily used for CAD Integrations to resolve there
        internal part to part links that Teamcenter would not know about. The RAC Duplicate Dialog does not use this
        option.
        """
        return cls.execute_soa_method(
            method_name='cloneStructure',
            library='StructureManagement',
            service_date='2014_10',
            service_name='Structure',
            params={'inputs': inputs},
            response_cls=CloneStructureResponse,
        )

    @classmethod
    def cloneStructureAsync(cls, inputs: List[CloneStructureInputInfo]) -> None:
        """
        This operation validates and creates a duplicate (clone) of the input structure from its top level down or a
        selected sub assembly. This operation runs asynchronously in its own server in the background.
        
        Depending on the input arguments, all or some of the original structure is duplicated and the rest is
        referenced, revised or replaced.
        
        The caller can define a specific naming pattern for the Item ids of the duplicated (cloned) structure. The
        caller can define specific Item ids for individually selected ItemRevision objects or a default naming pattern
        for the duplicated structure. The default pattern can be defined by adding prefixes, suffixes or replacing part
        of the original name with a different pattern. The caller can also choose to allow the system to assign default
        ids.
        
        If project(s) have been passed in as input, the cloned structure is assigned to that project(s). All of the
        structure dependent data of the input structure like Datasets and attachments are copied to the duplicated
        structure based on the Business Modeler IDE Deep Copy Rules for SaveAs or the Deep Copy Data override rules
        parameter passed in to the input structure. The duplicate operation internally uses SaveAs at every level of
        the structure; therefore it uses the SaveAs Deep Copy Rules.
        
        If the Systems Engineering/Requirements Manager structure, has tracelink GRMs they will be handled based on the
        deep copy rules set for ReqRev for SaveAs.
        
        CAD specific attachments and relations are copied based on the transfer mode defined for the Business Modeler
        IDE global constant StructureCloneTransferModes. The transfer mode contains dependent closure rules for
        expansion and cloning. The value for the closure rules is added by the installed CAD system. 
        
        Any errors during this operation will be captured and recorded in a Text dataset and attached to a Teamcenter
        Mail Envelop that will be sent to the user who initiated the process.
        
        Notes:
        
        
        
                -cloneStructureAsync was created as a result of the project to get NX CAD on board. The difference is
        cloneStructureAsync will now process set based inputs, it combined the validate and duplicate actions into one
        API, provides a validate only mode, introduces the ability to override DeepCopyData GRM rules to change the
        core default DeepCopy rules behavior (with exception of restricted rules), the ability to choose the folder to
        store new cloned root item revisions into, added 2 new action types revise and replace along with the reference
        and clone action types for determining what to do with the children of the structure being cloned.
        
        Use cases:
        A user has a very large assembly and wants to clone it in background mode so they can free up there client
        interface to perform other work. The user selects the assembly structure and selects the "Run In Background" 
        option.
        The system will then do the following:
        - A Message will be displayed to the user that the job was sent to the dispatcher.
        - The dispatcher will then send a Teamcenter Mail Envelope to the user informing them the Asynchronous
        Duplicate process has started. Note: if Teamcenter Mail is configured to send e-mails the user will then
        receive an e-mail stating the job has started.
        - The system will the start the validation and duplication of the structure.
        - When the duplicate process is done another Teamcenter Mail Envelope is sent to the user saying the duplicate
        process has completed. Also attached to the Mail Envelope is the new cloned toplevel item revision and a text
        dataset with information that was captured during the validate and duplicate process. Note: if Teamcenter Mail
        is configured to send e-mails the user will then receive an e-mail stating the job has completed.
        
        """
        return cls.execute_soa_method(
            method_name='cloneStructureAsync',
            library='StructureManagement',
            service_date='2014_10',
            service_name='Structure',
            params={'inputs': inputs},
            response_cls=None,
        )

    @classmethod
    def cloneStructureExpandOrUpdate(cls, opInput: List[CloneStructureExpandOrUpdateItemsInfo], selectionOption: int) -> CloneStructureExpandOrUpdateResponse:
        """
        This operation expand structures one level at a time and gets structure dependent data.
        
        When the 'selectionOption" is set to "1" for smart selection, it will try to solve the uncertain smart
        selection by expansion, in which case only qualified ItemRevision objects will be returned.
        
        The following are the CAD Dependency options the user can use when expanding or Updating a structure to be
        cloned.
        - Drawings
        - Required
        - PartFamilyMaster
        - PartFamilyMember
        - AllDep
        - Internal
        
        
        
        The CAD Dependency options correspond with the CAD Dependency rules defined by the Business Modeler IDE global
        constant "StructureCloneTransferModes". The CAD specific rules defined in the "StructureCloneTransferModes"
        will determine which of the CAD specific attachments and relationships can be expanded and included as part of
        the structure to be cloned. The values for the closure rules is added by the installed CAD system.
        
        Note: The differences between expandOrUpdateDuplicateItems3 and cloneStructureExpandOrUpdate are as follows:
        - The CAD options passed into this operation are now strings and no longer integers.
        - The cloneStructureExpandOrUpdate API calls a DuplicateExpandOrUpdate Business Object operation.The new
        Business Object operation allows CAD integrations to register extension code to identify CAD specific
        "Internal" relations to a structure being duplicated that are not published to Teamcenter.
        
        
        
        Use cases:
        Use case 1: selectionOption is 0 and the original structure has CAD data:
        
        The user sends in a structure for expansion, it will be expanded one level at a time and all dependent data
        will be returned based on the input and the value of the defined closure rule. The input consists of the
        BOMLine for expansion and ItemRevision objects on which to perform the expansion, the dependency types, and the
        selectionOption. The ItemRevision objects could be null, in which case the ItemRevision object(s) gotten from
        the expansion of the BOMLine are used. The dependency types are checked against the definition in the closure
        rules to determine what dependent data is expanded. 
        
        Use case 2: selectionOption is 1 and the original structure has no CAD data
        
        The user sends in a structure for expansion, it will be expanded one level at a time and all dependent data
        will be returned based on the input. In this case no closure rule may be defined, since the structure has no
        CAD data. The input consists of the BOMLine for expansion and ItemRevision objects on which to perform the
        expansion, the dependency types, and the selectionOption. The ItemRevision objects could be null, in which case
        the ItemRevision object(s) gotten from the expansion of the BOMLine are used. Since the selectionOption is 1,
        the input lines will be checked based on the top.
        """
        return cls.execute_soa_method(
            method_name='cloneStructureExpandOrUpdate',
            library='StructureManagement',
            service_date='2014_10',
            service_name='Structure',
            params={'opInput': opInput, 'selectionOption': selectionOption},
            response_cls=CloneStructureExpandOrUpdateResponse,
        )

    @classmethod
    def toggleOccurrenceSuppression(cls, inputs: List[BOMLine]) -> ServiceData:
        """
        This operation toggles occurrence suppression of the selected lines. 
        
        Use cases:
        User wants to suppress some lines after the structure is constructed, user can call this operation to toggle
        the occurrence suppression of the lines.
        """
        return cls.execute_soa_method(
            method_name='toggleOccurrenceSuppression',
            library='StructureManagement',
            service_date='2014_10',
            service_name='Structure',
            params={'inputs': inputs},
            response_cls=ServiceData,
        )

    @classmethod
    def togglePrecision(cls, inputs: List[BOMLine]) -> ServiceData:
        """
        This operation toggles precision of all lines.
        Note: 
        -    leaf lines cannot change precision.
        -    If multiple lines for same item revision are passed in to the operation, the precision for the structure
        of the lines will be changed only once. 
        
        
        Use cases:
        User wants to change precision of some lines after the structure is constructed, user can call this operation
        to toggle the precisions of the lines.
        """
        return cls.execute_soa_method(
            method_name='togglePrecision',
            library='StructureManagement',
            service_date='2014_10',
            service_name='Structure',
            params={'inputs': inputs},
            response_cls=ServiceData,
        )



from __future__ import annotations

from tcsoa.gen.Manufacturing._2013_05.ImportExport import AdvancedImportInput, AdvancedImportResponse
from tcsoa.gen.BusinessObjects import BusinessObject, WorkspaceObject
from tcsoa.gen.Manufacturing._2013_05.IPAManagement import IPAExistResponse, CleanIPATreeResponse, IPATreeInput, LocalUpdateIPATreeResponse, GenerateIPATreeResponse
from tcsoa.gen.Manufacturing._2009_10.DataManagement import CreateIn, CreateResponse
from tcsoa.gen.Manufacturing._2013_05.StructureManagement import FindAffectedCCsResponse
from tcsoa.gen.Manufacturing._2013_05.Core import MatchObjectsAgainstVariantRulesResponse, MatchObjectsAgainstVariantRulesArg, FindNodeInContextInputInfo
from tcsoa.gen.Manufacturing._2010_09.Core import FindNodeInContextResponse
from tcsoa.gen.Manufacturing._2013_05.StructureSearch import FindStudiesResponse
from typing import List
from tcsoa.gen.Manufacturing._2013_05.DataManagement import ObjectAttributesInput, SyncStudyResponse, SyncStudyInput
from tcsoa.gen.Manufacturing._2013_05.TimeWayPlan import TwpResponse, TwpInfoInput, ProductImageInfo
from tcsoa.gen.Server import ServiceData
from tcsoa.utils import TcService


class TimeWayPlanService(TcService):

    @classmethod
    def getTWPInformation(cls, input: TwpInfoInput) -> TwpResponse:
        """
        This service operation fetches the Time Way Plan (TWP) information. The operation takes objects from the plant
        Bill of Processes (BOP) for which TWP information is required, the list of strings specifying what information
        is required and the context product BOP. The object can be process station(s), process line(s), process
        area(s), or plant BOP. The list of string will have values "OperationDetails", "ExecutionPositions",
        "ProductDiagram", and "PlantCarpet" based on which response will contain the information. These values will be
        applicable to all the objects for which TWP information is required.
        
        This information is used to display the Time Way Plan view.
        """
        return cls.execute_soa_method(
            method_name='getTWPInformation',
            library='Manufacturing',
            service_date='2013_05',
            service_name='TimeWayPlan',
            params={'input': input},
            response_cls=TwpResponse,
        )

    @classmethod
    def setProductImage(cls, input: List[ProductImageInfo]) -> ServiceData:
        """
        This service operation sets a product image for the given input object. The operation takes objects from the
        plant Bill Of Processes (BOP) for which the product image is to be set, the business object of the context
        product BOP, and the business object of the dataset representing the product image. The object   from the plant
        BOP for which product image is to be set can be process station(s), process line(s), process area(s), or plant
        BOP. 
        This operation will create a relation between the object from the plant BOP input object and  the dataset
        business object in context of the input product BOP.
        The specified product image is displayed on the station in the Time Way Plan (TWP) view.
        """
        return cls.execute_soa_method(
            method_name='setProductImage',
            library='Manufacturing',
            service_date='2013_05',
            service_name='TimeWayPlan',
            params={'input': input},
            response_cls=ServiceData,
        )


class IPAManagementService(TcService):

    @classmethod
    def cleanIPATree(cls, processWindow: BusinessObject, forceCleanAll: bool) -> CleanIPATreeResponse:
        """
        This operation removes the In-Process Assembly (IPA) occurrences from the process structure and deletes the IPA
        occurrence group from product structure. This operation cleans only the IPA which has been configured by
        current configuration rule and configuring structure.
        """
        return cls.execute_soa_method(
            method_name='cleanIPATree',
            library='Manufacturing',
            service_date='2013_05',
            service_name='IPAManagement',
            params={'processWindow': processWindow, 'forceCleanAll': forceCleanAll},
            response_cls=CleanIPATreeResponse,
        )

    @classmethod
    def localUpdateIPATree(cls, processLines: List[BusinessObject]) -> LocalUpdateIPATreeResponse:
        """
        This operation is used to perform a local update on an In-Process Assembly (IPA) tree. Local update changes the
        IPA on which the operation is invoked. If necessary, it also updates the occurrence pointing to this in-process
        assembly in the process structure.
        
        If the IPA has not been created and attached to the matching process, Teamcenter also changes the matching
        process and adds the incoming IPA as MEWorkpiece. This happens only if a process whose type is in the process
        types list of the configuration details during the initial IPA generation.
        """
        return cls.execute_soa_method(
            method_name='localUpdateIPATree',
            library='Manufacturing',
            service_date='2013_05',
            service_name='IPAManagement',
            params={'processLines': processLines},
            response_cls=LocalUpdateIPATreeResponse,
        )

    @classmethod
    def updateIPATree(cls, ipaInput: IPATreeInput) -> GenerateIPATreeResponse:
        """
        This operation is used to update an In-Process Assembly (IPA) tree that already exists, when a process
        structure is changed. The options set at creation of the assembly tree may be changed for example process
        types, consumed occurrence types, name of the report on errors and problems, e-mail notification configuration.
        """
        return cls.execute_soa_method(
            method_name='updateIPATree',
            library='Manufacturing',
            service_date='2013_05',
            service_name='IPAManagement',
            params={'ipaInput': ipaInput},
            response_cls=GenerateIPATreeResponse,
        )

    @classmethod
    def doesIPAExist(cls, processWindow: BusinessObject) -> IPAExistResponse:
        """
        This operation checks if an In-Process Assembly tree exists for a process structure.
        """
        return cls.execute_soa_method(
            method_name='doesIPAExist',
            library='Manufacturing',
            service_date='2013_05',
            service_name='IPAManagement',
            params={'processWindow': processWindow},
            response_cls=IPAExistResponse,
        )

    @classmethod
    def generateIPATree(cls, ipaInput: IPATreeInput) -> GenerateIPATreeResponse:
        """
        An In-Process Assembly (IPA) is an aggregation of incoming parts into stations. IPA is generated based on
        consumed item(s) from a product structure in a process structure. Teamcenter creates a tree structure
        representing the IPA by traversing the process structure and collecting IPAs from previous process elements and
        adding the selected line's consumed objects. 
        
        The IPA is stored as an occurrence group and is displayed in a separate tab beside the base view of the root
        product.
        
        When the assembly tree is created by the operation, it places a configuration file as an attachment on the
        process for which the IPA is generated. 
        
        Teamcenter sends an e-mail notification to the recipients specified in the input after completion of the
        operation. This e-mail contains information about the IPA creation, as well as log files that are created
        during generation of the IPA.
        """
        return cls.execute_soa_method(
            method_name='generateIPATree',
            library='Manufacturing',
            service_date='2013_05',
            service_name='IPAManagement',
            params={'ipaInput': ipaInput},
            response_cls=GenerateIPATreeResponse,
        )


class ImportExportService(TcService):

    @classmethod
    def importManufacturingFeatures(cls, input: AdvancedImportInput) -> AdvancedImportResponse:
        """
        This service operation imports the discrete manufacturing features (MFGs) from a PLMXML file into a target
        product structure in Teamcenter.        This operation takes a scope line (container) as additional input and
        imports the MFGs under the container.        Moreover, it takes the import mode as input which allows deciding
        whether MFGs already present under the chosen container should be deleted or not.
        """
        return cls.execute_soa_method(
            method_name='importManufacturingFeatures',
            library='Manufacturing',
            service_date='2013_05',
            service_name='ImportExport',
            params={'input': input},
            response_cls=AdvancedImportResponse,
        )


class CoreService(TcService):

    @classmethod
    def matchObjectsAgainstVariantRules(cls, args: List[MatchObjectsAgainstVariantRulesArg]) -> MatchObjectsAgainstVariantRulesResponse:
        """
        This operation takes a list of pairs of runtime object and variant rule lists and determines for each
        object/variant rule combination of each pair whether the object is configured-in for the specified variant
        rule. Thereby it takes all aspects of an object into account that determine the visibility of the object, such
        as the variant conditions of the object itself and of all its parent objects as well as the conditions of any
        assigned object and that of its respective parent objects. The results will depend on the configuration state
        for IC, revision rule, effectivity etc of the implied windows, including reference windows.
        If a variant rule supplied in the arguments list is not matched by the variant configuration of any involved
        window a warning message will be added to the response structure, which indicates that the visibility check
        regarding the specific variant rule cannot be reliably performed because the configuration of the window
        contradicts the variant rule. 
        This operation currently does not support modular variants. Only saved variant rules (business object
        VariantRule) for the classic variant model are accepted or alternatively,  product variants (type
        Mfg0BvrProductVariant) which are linked to VariantRule objects. 
        """
        return cls.execute_soa_method(
            method_name='matchObjectsAgainstVariantRules',
            library='Manufacturing',
            service_date='2013_05',
            service_name='Core',
            params={'args': args},
            response_cls=MatchObjectsAgainstVariantRulesResponse,
        )

    @classmethod
    def findNodeInContext(cls, input: List[FindNodeInContextInputInfo]) -> FindNodeInContextResponse:
        """
        Getting the related objects of the selected object from the target contexts according to the input information.
        """
        return cls.execute_soa_method(
            method_name='findNodeInContext',
            library='Manufacturing',
            service_date='2013_05',
            service_name='Core',
            params={'input': input},
            response_cls=FindNodeInContextResponse,
        )


class DataManagementService(TcService):

    @classmethod
    def setAttributes(cls, input: List[ObjectAttributesInput]) -> ServiceData:
        """
        This service operation sets the attributes of objects attached to the business object(s). For example, if some
        attributes of a Form attached to an Item needs to be edited, this operation can be used. 
        The operation considers the Incremental Change applied on the window of the object whose attachment is to be
        edited.
        The operation takes the business objects of the BOMLine and its attached object which is to be edited. Along
        with that, it takes the name(s) of attributes  and their corresponding values to be set.
        This operation can set multiple attributes of multiple attached objects.
        """
        return cls.execute_soa_method(
            method_name='setAttributes',
            library='Manufacturing',
            service_date='2013_05',
            service_name='DataManagement',
            params={'input': input},
            response_cls=ServiceData,
        )

    @classmethod
    def syncStudyAndSource(cls, input: List[SyncStudyInput]) -> SyncStudyResponse:
        """
        This operation synchronizes a Simulation Study with the source BOP structure it is associated with.
        """
        return cls.execute_soa_method(
            method_name='syncStudyAndSource',
            library='Manufacturing',
            service_date='2013_05',
            service_name='DataManagement',
            params={'input': input},
            response_cls=SyncStudyResponse,
        )

    @classmethod
    def createAttachments(cls, input: List[CreateIn]) -> CreateResponse:
        """
        This service operation creates the attachment objects for a business object(s). 
        The operation considers the Incremental Change applied on the window of the object for which attachment is to
        be created.
        The operation takes following inputs.
        - clientId - Unique client Identifier.
        - context - The business objects of the BOM Line. The IC applied on the window of this line is considered while
        creating the attachment. 
        - target - The business object which is used as primary object to connect the newly created object. 
        - relation name - The name of the relation used to connect the target. 
        - data - Input data for create operation.
        
        """
        return cls.execute_soa_method(
            method_name='createAttachments',
            library='Manufacturing',
            service_date='2013_05',
            service_name='DataManagement',
            params={'input': input},
            response_cls=CreateResponse,
        )


class StructureManagementService(TcService):

    @classmethod
    def findAffectedCCs(cls, objects: List[WorkspaceObject]) -> FindAffectedCCsResponse:
        """
        This operation finds all of the Collaboration Contexts which contain a process structure, which contains the
        input objects (e.g. Item, MEWorkArea, METool, MEProcess or MEOperation). The input objects must be of type Item
        or ItemRevision (BOMLine objects are not valid input objects).
        
        Exceptions:
        >Throws:
        NotSupportedObjectType: If the input object isn't one of the supported types (it's not a subtype of an item or
        an item_revision).
        """
        return cls.execute_soa_method(
            method_name='findAffectedCCs',
            library='Manufacturing',
            service_date='2013_05',
            service_name='StructureManagement',
            params={'objects': objects},
            response_cls=FindAffectedCCsResponse,
        )


class StructureSearchService(TcService):

    @classmethod
    def findStudies(cls, inputNodes: List[BusinessObject]) -> FindStudiesResponse:
        """
        This operation finds all study objects that reference a given process/operation business object.
        """
        return cls.execute_soa_method(
            method_name='findStudies',
            library='Manufacturing',
            service_date='2013_05',
            service_name='StructureSearch',
            params={'inputNodes': inputNodes},
            response_cls=FindStudiesResponse,
        )



from __future__ import annotations

from tcsoa.gen.Internal.Core._2014_10.FileManagement import DatashareManagerDownloadInfo, DatashareManagerUploadInfo, GetPlmdFileTicketResponse
from tcsoa.gen.BusinessObjects import ImanFile
from tcsoa.gen.Internal.Core._2014_10.Licensing import LicenseServerInput
from typing import List
from tcsoa.gen.Server import ServiceData
from tcsoa.utils import TcService


class FileManagementService(TcService):

    @classmethod
    def postCleanUpFileCommits(cls, files: List[ImanFile]) -> ServiceData:
        """
        This operation recommits the file  after it has been cleaned out from local volume by the File Management
        System (FMS) StoreAndForward feature. This operation is intended for use only by the DatabaseOperations()
        method of an FMSTransferTool transfer task running in a Dispatcher Client context.
        The File Management System (FMS) StoreAndForward feature redirects uploads to a temporary volume that is close
        to the uploading user, called a Default Local Volume. The system then transfers the file to its final home in a
        Default Volume in a background FMSTransferTool process executed on a Dispatcher Module.
        The first step of the background operation is to copy the file(s) from the Default Local Volume to the Default
        Volume and  commit the new volumes of each file to the Teamcenter database. The second step of the background
        operation is to clean up the files from local volume. Finally the task invokes this operation to re-commit the
        destination volume to null for each file to the Teamcenter database.
        This operation is unpublished.  It is supported only for internal Siemens PLM purposes.  Customers should not
        invoke this operation.
        """
        return cls.execute_soa_method(
            method_name='postCleanUpFileCommits',
            library='Internal-Core',
            service_date='2014_10',
            service_name='FileManagement',
            params={'files': files},
            response_cls=ServiceData,
        )

    @classmethod
    def getPlmdFileTicketForDownload(cls, infos: List[DatashareManagerDownloadInfo]) -> GetPlmdFileTicketResponse:
        """
        The operation generates a File Management System (FMS) transient read ticket for Product Lifecycle Management
        Data (PLMD) file. The PLMD file is used by Data Share Manager Application to asynchronously download the files
        from a Teamcenter volume to a local client environment. The PLMD file contains details like FMS Bootstrap URLs,
        dataset names, type, reference names, original file names, and absolute file paths and read tickets.
        This operation is unpublished.  It is supported only for internal Siemens PLM purposes.  Customers should not
        invoke this operation.
        """
        return cls.execute_soa_method(
            method_name='getPlmdFileTicketForDownload',
            library='Internal-Core',
            service_date='2014_10',
            service_name='FileManagement',
            params={'infos': infos},
            response_cls=GetPlmdFileTicketResponse,
        )

    @classmethod
    def getPlmdFileTicketForUpload(cls, infos: List[DatashareManagerUploadInfo]) -> GetPlmdFileTicketResponse:
        """
        The operation generates a File Management System (FMS) transient read ticket for Product Lifecycle Management
        Data (PLMD) file. The PLMD file is used by Data Share Manager Application to asynchronously upload the files
        from a  local client environment into a Teamcenter volume. The PLMD file contains details like FMS Bootstrap
        URLs, dataset names, type, reference names, original file names, and absolute file paths and read tickets.
        This operation is unpublished.  It is supported only for internal Siemens PLM purposes.  Customers should not
        invoke this operation.
        """
        return cls.execute_soa_method(
            method_name='getPlmdFileTicketForUpload',
            library='Internal-Core',
            service_date='2014_10',
            service_name='FileManagement',
            params={'infos': infos},
            response_cls=GetPlmdFileTicketResponse,
        )


class LicensingService(TcService):

    @classmethod
    def updateLicenseServer(cls, inputObjects: List[LicenseServerInput]) -> ServiceData:
        """
        This operation modifies the Fnd0LicenseServer business object for each 'LicenseServerInput' supplied. The
        'LicenseServerInput' structure contains information for properties such as license server name, host, port,
        protocol and failover servers, for a given license server.  A license server location is defined by its host
        and port.  It is not allowed to have two license server names pointing to the same license server location. The
        user performing the operation needs administrator privileges.
        """
        return cls.execute_soa_method(
            method_name='updateLicenseServer',
            library='Internal-Core',
            service_date='2014_10',
            service_name='Licensing',
            params={'inputObjects': inputObjects},
            response_cls=ServiceData,
        )



from __future__ import annotations

from tcsoa.gen.ProjectManagementAw._2016_12.ScheduleManagementAw import EventDateRanges
from tcsoa.gen.BusinessObjects import TaskDependency, Schedule, Fnd0ProxyTask, WorkspaceObject, ScheduleTask
from typing import Dict, List
from tcsoa.gen.Server import ServiceData
from tcsoa.utils import TcBaseObj
from dataclasses import dataclass


@dataclass
class BaselineTaskInfo(TcBaseObj):
    """
    Contains information about the baselineTask.
    
    :var baselineTask: The baseline ScheduleTask object.
    :var properties: Map (string, string) of baseline task property names and their values.
    """
    baselineTask: ScheduleTask = None
    properties: StringToStringMap = None


@dataclass
class LoadBaselineInfo(TcBaseObj):
    """
    Information required to load the baseline tasks of a Schedule baseline based on a source Schedule and load options.
    
    :var sourceSchedule: The Schedule for which the baseline needs to be loaded.
    :var baselineSchedule: The baseline Schedule to load.
    :var scheduleTasks: The list of ScheduleTask objects, in the source Schedule, for which the respective baseline
    tasks are to be returned.
    :var loadOptions: Specifies the options for loading Schedule Baseline. Valid options (key : value) are:
    loadBaselineTasks : true/false (Set true to return the baseline tasks; false otherwise)
    loadCompleteBaseline : true/false (Set true to return information of all the baseline tasks in the schedule
    baseline; Set false or do not specify this option to return the baseline task information of only the input
    ScheduleTask objects.)
    """
    sourceSchedule: Schedule = None
    baselineSchedule: Schedule = None
    scheduleTasks: List[ScheduleTask] = ()
    loadOptions: StringToStringMap = None


@dataclass
class LoadBaselineResponse(TcBaseObj):
    """
    Response of the load baseline operation, containing information about the loaded baseline tasks.
    
    :var baselineTasksInfo: Map (string, BaselineTaskInfo) of original ScheduleTask UID and its baseline task
    information.
    :var serviceData: The ServiceData.
    """
    baselineTasksInfo: TaskUidToBaselineTaskInfoMap = None
    serviceData: ServiceData = None


@dataclass
class LoadScheduleInfo2(TcBaseObj):
    """
    Information required to load a Schedule based on a reference ScheduleTask, parents of the reference task and
    schedule load options.
    
    :var schedule: The Schedule to load.
    :var referenceTask: The reference ScheduleTask from where to start loading the subsequent ScheduleTask objects,
    which could be either siblings of reference ScheduleTask or siblings of one of the parents of reference
    ScheduleTask. If the reference ScheduleTask is null, the children of the parentTasks specified in the input will be
    loaded.
    :var parentTasks: List of all top level parents of the referenceTask, with immediate parent at the start of the
    list. If not specified, no ScheduleTask objects will be loaded.
    :var loadOptions: Specifies the options for loading a Schedule. Valid options (key : value) are:
    baselineUid : UID of the schedule baseline
    loadBaselineTasks : true/false (Set true to return the baseline tasks; false otherwise)
    loadCalendarInfo : true/false (Set true to return the calendar information; false otherwise)
    """
    schedule: Schedule = None
    referenceTask: ScheduleTask = None
    parentTasks: List[ScheduleTask] = ()
    loadOptions: StringToStringMap = None


@dataclass
class LoadScheduleResponse2(TcBaseObj):
    """
    Response of the load schedule operation, containing the list of loaded ScheduleTask, parent ScheduleTask, baseline
    ScheduleTask, Fnd0ProxyTask and TaskDependency objects. Also, contains the calendar information of the loaded
    Schedule.
    
    :var scheduleTasksInfo: Information about the ScheduleTask objects loaded.
    :var proxyTasks: List of Fnd0ProxyTask objects referencing the loaded ScheduleTask objects.
    :var taskDependenciesInfo: Information about the Task Dependency objects loaded.
    :var hasMoreTasks: If true, there are more ScheduleTask objects in the Schedule, which are not loaded by this
    operation.
    :var taskBaselineMap: Map (ScheduleTask, BaselineTaskInfo) containing the loaded ScheduleTask objects and their
    respective baseline tasks information. If a loaded ScheduleTask object does not have a baseline task, it will not
    be added to the map.
    :var calendarInfo: Calendar information of the loaded Schedule. It contains details about the standard working
    hours and holidays within the Schedule boundaries.
    :var serviceData: The ServiceData.
    """
    scheduleTasksInfo: List[ScheduleTaskInfo] = ()
    proxyTasks: List[Fnd0ProxyTask] = ()
    taskDependenciesInfo: List[TaskDependencyInfo] = ()
    hasMoreTasks: bool = False
    taskBaselineMap: BaselineTasks = None
    calendarInfo: CalendarInfo = None
    serviceData: ServiceData = None


@dataclass
class CalendarInfo(TcBaseObj):
    """
    Contains calendar information of a Schedule object.
    
    :var dayRanges: Map (int, list of string) containing the ranges for each day in a week. Maps the days of the week
    to their ranges. The valid values for days are 0 to 6 which represents Sunday through Saturday in the respective
    order. A range is a pair of start and end time of working hours. A day can have multiple ranges with breaks
    in-between in a working day.
    :var eventRanges: The event dates and their respective ranges. The events are exceptions where the working hours
    are different from the default working hours.
    """
    dayRanges: DayRanges2 = None
    eventRanges: List[EventDateRanges] = ()


@dataclass
class RecalculateInput(TcBaseObj):
    """
    A list of structures containing information to recalculate schedule.
    
    :var schedule: The Schedule to be recalculated.
    :var recalcType: The recalculation type: -1-All, 1-Execution Data, 2-Scheduling Data.
    """
    schedule: Schedule = None
    recalcType: int = 0


@dataclass
class ScheduleTaskInfo(TcBaseObj):
    """
    Contains a ScheduleTask object and its parent ScheduleTask object.
    
    :var scheduleTask: The ScheduleTask object in context.
    :var parentTask: The parent of the scheduleTask. It will be null if the scheduleTask is of Schedule Summary Task
    type.
    """
    scheduleTask: ScheduleTask = None
    parentTask: ScheduleTask = None


@dataclass
class TaskDeliverableInput(TcBaseObj):
    """
    The input information required for creating task deliverables.
    
    :var schedule: The owning Schedule of the Schedule Task objects.
    :var scheduleTasks: The list of Schedule Task objects in the schedule for which the deliverables are to be created.
    :var submitType: The type of submission of workflow to which the deliverable is routed. Valid values are 3=Don't
    submit, 0=submit as target, 1=submit as reference.
    :var deliverableReference: The deliverable instance to be referenced by the Schedule deliverable.
    :var deliverableName: The name of the deliverable.
    :var deliverableType: The type of the instance associated with the deliverable (Item, Form, Folder, Dataset, and
    Document)
    """
    schedule: Schedule = None
    scheduleTasks: List[ScheduleTask] = ()
    submitType: int = 0
    deliverableReference: WorkspaceObject = None
    deliverableName: str = ''
    deliverableType: str = ''


@dataclass
class TaskDependencyInfo(TcBaseObj):
    """
    Contains information about the TaskDependency object.
    
    :var taskDependency: The Task Dependency object.
    :var properties: Map (string, string) of task dependency property names and their values
    """
    taskDependency: TaskDependency = None
    properties: StringToStringMap = None


"""
Map of ScheduleTask object and its baseline task information.
"""
BaselineTasks = Dict[ScheduleTask, BaselineTaskInfo]


"""
Maps the weekdays to their ranges. The valid values for the days are 0 to 6 which represents Sunday through Saturday in the respective order. A range is a pair of start and end time of working hours. A day can have multiple ranges with breaks in-between in a working day.
"""
DayRanges2 = Dict[int, List[str]]


"""
Map (string, string) containg a key value pair of string type.
"""
StringToStringMap = Dict[str, str]


"""
Map (string, BaselineTaskInfo) of original ScheduleTask UID and its baseline task information.
"""
TaskUidToBaselineTaskInfoMap = Dict[str, BaselineTaskInfo]



from __future__ import annotations

from typing import Dict, List
from tcsoa.gen.Server import ServiceData
from tcsoa.utils import TcBaseObj
from dataclasses import dataclass


@dataclass
class FetchOdsResponse(TcBaseObj):
    """
    The return structure for the fetchOdsRecords operation.
    
    :var serviceData: The ServiceData. Returns partial failure information.
    :var results: The results structure for the fetchOdsRecords operation.
    """
    serviceData: ServiceData = None
    results: FetchOdsResults = None


@dataclass
class FetchOdsResults(TcBaseObj):
    """
    Stores the returned data from this operation.
    
    :var systemLogFiles: A list of the names of the remote system log files used to assist in troubleshooting.
    :var fmsTickets: A list of FMS read tickets to the TCXML output files.
    :var failedObjects: A list of failed Business Objects UIDs.
    :var odsSessionOutput: Data returned from the current ODS session.
    """
    systemLogFiles: List[str] = ()
    fmsTickets: List[str] = ()
    failedObjects: List[str] = ()
    odsSessionOutput: StringVectorMap = None


"""
Map of string array property names to values (string, vector).
"""
StringVectorMap = Dict[str, List[str]]



from __future__ import annotations

from tcsoa.gen.BusinessObjects import BusinessObject
from typing import Dict, List
from tcsoa.gen.Workflow._2008_06.Workflow import Resources
from tcsoa.gen.Server import ServiceData
from tcsoa.utils import TcBaseObj
from dataclasses import dataclass


@dataclass
class GetRegisteredHandlerResponse(TcBaseObj):
    """
    Returns list of registered action and rule handler names.
    
    :var actionHandlers: A list of names of registered action handlers.
    :var ruleHandlers: A list of names of registered rule handlers.
    :var serviceData: The service data.
    """
    actionHandlers: List[str] = ()
    ruleHandlers: List[str] = ()
    serviceData: ServiceData = None


@dataclass
class CreateOrUpdateHandlerResponse(TcBaseObj):
    """
    Response of createOrUpdateHandler operation.
    
    :var createdorUpdatedObjects: A list of CreateUpdateHandlerOutput structure.
    :var serviceData: The service data.
    """
    createdorUpdatedObjects: List[CreateUpdateHandlerOutput] = ()
    serviceData: ServiceData = None


@dataclass
class CreateOrUpdatePALResponse(TcBaseObj):
    """
    Response structure returned by createOrUpdatePAL operation.
    
    :var createdorUpdatedObjects: A list of CreateUpdatePALOutput structure.
    :var serviceData: The service data.
    """
    createdorUpdatedObjects: List[CreateUpdatePALOutput] = ()
    serviceData: ServiceData = None


@dataclass
class CreateOrUpdateTemplateResponse(TcBaseObj):
    """
    Response of createOrupdate operation.
    
    :var createdorUpdatedObjects: A list of CreateUpdateTemplateOutput structure.
    :var serviceData: The service data.
    """
    createdorUpdatedObjects: List[CreateUpdateTemplateOutput] = ()
    serviceData: ServiceData = None


@dataclass
class CreateUpdateHandlerInput(TcBaseObj):
    """
    Input structure to pass requried data for create or update handler.
    
    :var clientID: A unique string supplied by the caller. This ID is used to identify return data elements and partial
    errors associated with this input structure.
    :var handlerName: Name of the handler to be created. This name represents registered workflow handlers only. For
    example: If User want to create a new action handler named "EPM-set-status" then user need to pass same name in
    this input.This input is required for create case.
    :var taskTemplate: UID of EPMTaskTemplate object on which handler is to be created or updated.
    :var businessRule: UID of EPMBusinessRule object.This object represents the workflow business rule.This input is
    required for create case ( creating new Rule Handler). For create case if the handlerType input is set to "Rule" (
    user is creating new Rule Handler ) then user don&rsquo;t need to pass this input and SOA operation will create its
    own new object of EPMBusinessRule and newly created workflow handler instance ( EPMHandler ) will be attached to it.
    :var handlerType: Type of the handler to be created. Required input for create and the supported values are:
    "Rule"    : Creates rule handler.
    "Action" : Creates action handler.
    :var handlerToUpdate: UID of the EPMHandler object to be updated. If this  input is provided then it will be
    considered as update case.
    :var action: The action of the workflow task on which new handler will be added. Required for create.
    Supported values are:
     Action    Value
    Assign          1
    Start              2
    Complete  4
    Promote      5
    Suspend      6
    Resume      7
    Undo          8
    Abort          9
    Perform     100
    :var ruleQuorum: The rule quorum value to specify whether one rule, all rules, or a number of rules must be
    satisfied for the task to progress. Below are the accepted values :
     -1 :  means all. In this case, every rule must pass to meet the quorum.
    A number greater than 1, but less than the number of already added Rule handler in the Rule container. Any invalid
    input will be ignored.
    :var changeExecutionOrder: The position of the handler in added handler(s) array on particularaction of the
    EPMTaskTemplate. For example specifying the -1 position moves the given handler (handlerToUpdate input) up in a
    handler array. Similarly specifying the 1 position moves the given handler down in a handler array. Any invalid
    input will be ignored.
    :var additionalData: A map (string, list of strings) to send additional information. This map is used to pass
    argument data to the handler to be created or updated.Handler arguments varies from handler to handler. User need
    to pass the requried/optional arguments as documented in Teamcenter workflow handler document. Below is an example
    of EPM-set-status handler:
    
    Key : -action     
    Values : {"append"}
    Description : Support values are:"append", "rename", "replace", or delete.
    
    Key : -status     
    Values : {"name_of_status"}
    Description : Specifies the name of the release status.
    """
    clientID: str = ''
    handlerName: str = ''
    taskTemplate: str = ''
    businessRule: str = ''
    handlerType: str = ''
    handlerToUpdate: str = ''
    action: int = 0
    ruleQuorum: int = 0
    changeExecutionOrder: int = 0
    additionalData: KeyValuesMap = None


@dataclass
class CreateUpdateHandlerOutput(TcBaseObj):
    """
    Structure represents the output of createOrUpdateHandler operation.
    
    :var clientID: A unique string supplied by the caller. This ID is used to identify  return data elements and
    partial errors associated with the input  structure.
    :var handlerObject: Updated or created EPMHandler object.
    :var ruleObject: Updated or created EPMBusinessRule object.
    """
    clientID: str = ''
    handlerObject: BusinessObject = None
    ruleObject: BusinessObject = None


@dataclass
class CreateUpdatePALInput(TcBaseObj):
    """
    Input stucture used to update or copy EPMAssignmentList or to create a new EPMAssignmentList.
    
    :var clientID: A unique string supplied by the caller. This ID is used to identify return data elements and partial
    errors associated with this input structure.
    :var palName: Name of the EPMAssignmentList to be updated or newly created . In case of copy operation, this is the
    name of new EPMAssignmentList to be created.
    :var workflowTemplate: UID of EPMTaskTemplate object for which process assignment list is to be created or updated
    or copied.
    :var palDescription: Description of process assignment list.
    :var resourceLists: A list of Resources to be assigned.
    :var additionalData: A map (string, list of strings) to send additional information. Currently supported keys and
    values are:
    
    - Key:  "isShared"
    - Values :  {"true", "false"}
    - Description: If true, a shared assignment list is created or updated. If this key values pair is not passed or
    &lsquo;false&rsquo;                                                 value is passed then the assignment list is not
    shared.
    
    
    
    - Key:  "palToUpdate"
    - Values : {"palUIDToUpdate"}
    - Description: &lsquo;palUIDToUpdate&rsquo; is the UID of EPMAssignmentList which has to be updated.
    
    
    
    - Key:  "palToCopy"
    - Values : {"palUIDToCopy"}
    - Description: &lsquo;palUIDToCopy&rsquo; is the UID of EPMAssignmentList which has to be copied.
    
    
    
    Keys &lsquo;palToUpdate&rsquo; and &lsquo;palToCopy&rsquo; are supported mutually exclusively. If both keys are
    provided, then error #219031 is returned. If none of the keys, &lsquo;palToUpdate&rsquo; and
    &lsquo;palToCopy&rsquo; are provided; then a new assignment list is created.
    """
    clientID: str = ''
    palName: str = ''
    workflowTemplate: str = ''
    palDescription: List[str] = ()
    resourceLists: List[Resources] = ()
    additionalData: KeyValuesMap = None


@dataclass
class CreateUpdatePALOutput(TcBaseObj):
    """
    CreateUpdatePALOutput structure.
    
    :var clientID: A unique string supplied by the caller. This ID is used to identify return data elements and partial
    errors associated with this input structure.
    :var palObject: Updated or created EPMAssignmentList object.
    """
    clientID: str = ''
    palObject: BusinessObject = None


@dataclass
class CreateUpdateTemplateInput(TcBaseObj):
    """
    Input structure used for passing requried data for create or update template object.
    
    :var clientID: A unique string supplied by the caller. This ID is used to identify return data elements and partial
    errors associated with this input structure.
    :var templateName: Name of the task template to be created or updated.
    :var templateDesc: Description of the task template to be created or updated.
    :var baseTemplate: UID of EPMTaskTemplate object which will be used as a base template.Basically new template to be
    created will be a exact copy of this template. This is requried for create operation.
    :var parentTemplate: UID of EPMTaskTemplate object represents the parent object. New template will be created as a
    child or sub template of this parent template input. This is requried for create operation.
    :var templateToUpdate: UID of the EPMTaskTemplate object to be updated. If this input is provided then this
    operation will be considered as update case.
    :var additionalData: A map (string, list of strings) to send additional data. This input is reserverd for future
    use.
    """
    clientID: str = ''
    templateName: str = ''
    templateDesc: str = ''
    baseTemplate: str = ''
    parentTemplate: str = ''
    templateToUpdate: str = ''
    additionalData: KeyValuesMap = None


@dataclass
class CreateUpdateTemplateOutput(TcBaseObj):
    """
    Struture holds the created or updated template object.
    
    :var clientID: A unique string supplied by the caller. This ID is used to identify return data elements and partial
    errors associated with the input structure.
    :var templateObject: Updated or created EPMTaskTemplate object.
    """
    clientID: str = ''
    templateObject: BusinessObject = None


"""
A map (string, list of strings) to send additional information.
"""
KeyValuesMap = Dict[str, List[str]]



from __future__ import annotations

from tcsoa.gen.ImportExport._2008_06.FileImportExport import ExportTemplateInput, ExportToApplicationInputData1, ImportFromApplicationInputData1, GetExportTemplateResponse
from tcsoa.gen.ImportExport._2007_06.FileImportExport import ExportToApplicationResponse, ImportFromApplicationResponse
from typing import List
from tcsoa.utils import TcService


class FileImportExportService(TcService):

    @classmethod
    def importFromApplication(cls, inputs: List[ImportFromApplicationInputData1]) -> ImportFromApplicationResponse:
        """
        This operation is used for importing the contents of the given MSWord document (.docx) to create objects of
        type SpecElement.  Based on the application format, this operation can also be used to import objects of type
        SpecTemplate,ObjectTemplate and ExcelTemplate in database. If the application format is "'MSWordXML'"then the
        operation parses the given MS Word document to creates a structure of type SpecElement.  If the application
        format is "'MSWordSpecTemplate'" or "'MSWordObjTemplate'"or "'MSExcelTemplate'" then the chosen template can be
        imported into Teamcenter.  If the application format is "'MSWordXMLLive'" then a "Live" document can be
        imported to Teamcenter.  To use this application format, a "Live" document should be generated from Teamcenter.
        If the application format is "'MSWordSetContent'" then it can set the rich text of the SpecElement  and the
        properties on the SpecElement by using the SpecficationTemplate and ObjectTemplate at the server. If the
        application format is "'MSWordXMLOverWriteCheck'" then the Teamcenter objects will be checked for overwrite
        condition at the server.
        The objects of type Requirement/Paragraph are created at the server after import of the document.  If this
        operation is called from Teamcenter rich client, a structure is created and is opened in the
        RequirementsManagement application.  The structure and indentation of the child Requirement is driven by the
        outline levels in the document.  Each Paragraph that is formatted in an outline level style produces a separate
        Requirement. This file may be located in a local file system folder or a network folder.  Import of document to
        Teamcenter supports Word documents in MS Office Open XML format (.docx format) only. Import of templates to
        Teamcenter supports MSWord documents in MS Office Open XML format. (.docx and .docm format)
        
        Use cases:
        User can create an MSWord document and import it using this operation to create a structure. Each paragraph in
        MSWord document represents a Requirement/Paragraph in the structure. User can import templates (SpecTemplate,
        ObjectTemplate and ExcelTemplate) to Teamcenter. User can import a "Live" document generated in Teamcenter and
        set the properties of Workspace objects. User can set the rich text of the SpecElement using the embedded Word
        from RAC. User can prevent overwrite of objects which are edited by others.
        
        Exceptions:
        >Service Exception If there is any error during generating transient file ticket due to a configuration issue
        at the server, then the operation throws a service exception. Example - If there is any exception due FMS not
        configured then the file upload to server will fail and a service exception will occur.
        """
        return cls.execute_soa_method(
            method_name='importFromApplication',
            library='ImportExport',
            service_date='2008_06',
            service_name='FileImportExport',
            params={'inputs': inputs},
            response_cls=ImportFromApplicationResponse,
        )

    @classmethod
    def exportToApplication(cls, inputs: List[ExportToApplicationInputData1]) -> ExportToApplicationResponse:
        """
        This operation is used for exporting Teamcenter objects (WorkspaceObject) to applications like MSWord and
        MSExcel. Teamcenter business objects should already be created so that the objects can be exported to MSWord
        and MSExcel using the service operation.
        An Excel (.mhtml) or a Word (.docx) file or a comma separated file (.csv) file is generated at the server as a
        result of this operation.  The generated file will contain Teamcenter objects and their properties. Depending
        upon the application format that is being passed as input parameter, the generated file can be opened in MSWord
        or MXExcel.  If the application format is "MSWordXML" then the generated document is a Word document.  It
        traverses the structure for the given SpecElement deep and exports all its children to MSWord document. If the
        application format is "MSExcel" then the generated sheet is a static Excel spreadsheet.  If the application
        format is "MSExcelLive" then the generated sheet is a Live Excel spreadsheet. "Live" sheet means that
        modifications can be done to the data will reflect to Teamcenter.  If the application format is "MSWordXMLLive"
        then the generated document is a Live Word document.  A "Live" Word diocument means that the any modifications
        done in the document like changing the rich text or editing of properties will be saved to Teamcenter on click
        of "Save" button in MSWord. If the application format is "MSExcelReimport" then the generated sheet can be
        reimported back to Teamcenter. If the application format is "MSExcelLiveBulkMode"then the sheet is generated in
        "Bulk Live" mode. This mode enables the user to make all the changes to the properties of objectsfrom Excel and
        then commit the changes to Teamcenter on click of "Save to Teamcenter" button in Excel.
        
        
        Use cases:
        User can create Teamcenter objects in RAC and then export those objects and their properties to Word or Excel.
        If "Live" option is selected then User can create "Live" documents after export to Word/Excel.  During the
        export to Word/Excel, Teamcenter objects are exported to Word/Excel by applying the appropriate templates. User
        can export Teamcenter objects to Excel which can be imported back to Teamcenter. User can create excel sheet in
        a "Bulk Live" mode so that bulk changes can be committed to Teamcenter.  User can export the objects and
        properties to a comma separated file. User can create SpecTemplate, ObjectTemplate and ExcelTemplate in
        Teamcenter and customize these templates as per User need such as adding more properties or adding formatting
        information to the templates.  User can associate different objectTemplate for every Business object type at
        runtime and specify this configuration when using this operation.
        
        Exceptions:
        >Service Exception  If there is any error during generating transient file read ticket due to a configuration
        issue at the server, then the operation throws a service exception. Example-  If the transient volume directory
        is not configured at the server then the FMS fails to generate a file at the server and subsequent file
        download operation fails. In such situation a service exception is thrown.
        """
        return cls.execute_soa_method(
            method_name='exportToApplication',
            library='ImportExport',
            service_date='2008_06',
            service_name='FileImportExport',
            params={'inputs': inputs},
            response_cls=ExportToApplicationResponse,
        )

    @classmethod
    def getExportTemplates(cls, filter: ExportTemplateInput) -> GetExportTemplateResponse:
        """
        This operation is used for retrieving all the MSWord and MSExcel templates from database. An input filter can
        be applied to get the desired template type from database. All the Word and Excel templates are of 'Item' type.
        Depending on the input filter, the Item of type SpecTemplate or ObjectTemplate or ExcelTemplate are retrieved
        from the database. These templates are used by the application for export purpose. If the chosen filter is of
        type SpecTemplate or ObjectTemplate then the document to be imported is in .docx or .docm file format.  If the
        chosen filter is of type ExcelTemplate then the spreadsheet to be imported is in .xlsx or .xlsm file format.
        
        Use cases:
        User can import MSWord and MSExcel templates to Teamcenter and use these templates for export purpose. During
        this operation, items of type SpecTemplate or ObjectTemplate or ExcelTemplate is created after the import
        operation in Teamcenter.
        """
        return cls.execute_soa_method(
            method_name='getExportTemplates',
            library='ImportExport',
            service_date='2008_06',
            service_name='FileImportExport',
            params={'filter': filter},
            response_cls=GetExportTemplateResponse,
        )



from __future__ import annotations

from tcsoa.gen.BusinessObjects import BusinessObject, Ac0Comment, Ac0Conversation, User
from typing import Dict, List
from tcsoa.gen.Server import ServiceData
from tcsoa.utils import TcBaseObj
from dataclasses import dataclass


@dataclass
class CreateOrUpdateCommentRequest(TcBaseObj):
    """
    The CreateOrUpdateCommentRequest is a data structure that contains the parameters required for the create or update
    comment request.
    
    :var richText: The original formatted text and content entered by the user using ckeditor.
    :var rootComment: The Ac0Comment  root comment object.  This will be null if it is the first comment on the
    conversation.
    :var comment: The Ac0Comment  object to be updated. This is empty if the comment is being created.
    :var conversation: The Ac0Conversation object this new or updated comment belongs to.
    """
    richText: str = ''
    rootComment: Ac0Comment = None
    comment: Ac0Comment = None
    conversation: Ac0Conversation = None


@dataclass
class CreateOrUpdateConversationRequest(TcBaseObj):
    """
    The CreateOrUpdateConversationRequest is a data structure that contains the parameters required for a create or
    update conversation request.
    
    :var sourceObjects: A list   containing BusinessObject objects  that are the conversation&rsquo;s source objects.  
     A sourceObject can be any WorkspaceObject users want to have a conversation about.
    :var listOfPartipants: A list  containing User objects that are the conversation&rsquo;s participants.  This field
    is null for public conversations. For private conversations this is a mandatory parameter.
    :var defaultCommentText: The original formatted text and content entered by the user using ckeditor.   This field
    contains the text of the initial comment on a conversation.
    :var conversation: The Ac0Conversation object if the conversation is being updated.  This parameter is null when
    the conversation is new.
    """
    sourceObjects: List[BusinessObject] = ()
    listOfPartipants: List[User] = ()
    defaultCommentText: str = ''
    conversation: Ac0Conversation = None


@dataclass
class ReadResponse(TcBaseObj):
    """
    The ReadResponse is a data structure that holds maps containing the requested users and objects where read
    privileges have not been granted.
    
    :var userObjectMap: The map (User, list of BusinessObject) of User entries that correspond to a list of
    WorkspaceObject objects for which the User does not have read permissions.  The map will be empty if all provided
    users have read access to all provided objects.
    :var objectUserMap: The map (BusinessObject, list of User) of WorkspaceObject entries that correspond to a list of
    User objects that do not have read permission for the given WorkspaceObject.
    :var serviceData: The Service Data.
    """
    userObjectMap: UserObjectMap = None
    objectUserMap: ObjectUserMap = None
    serviceData: ServiceData = None


"""
The objectUserMap contains objects and a corresponding list of users that do not have read access to the given object.
"""
ObjectUserMap = Dict[BusinessObject, List[User]]


"""
The userObjectMap contains user objects and a corresponding list of business objects that the given user does not have read access to.
"""
UserObjectMap = Dict[User, List[BusinessObject]]



from __future__ import annotations

from typing import List
from tcsoa.gen.Manufacturing._2019_06.DataManagement import SelectedSyncPublishStudyInput, SelectedStudySourceResponse
from tcsoa.utils import TcService


class DataManagementService(TcService):

    @classmethod
    def publishSelectionFromStudyToSource(cls, input: List[SelectedSyncPublishStudyInput]) -> SelectedStudySourceResponse:
        """
        This service operation publishes the changes from Mfg0BvrSimStudy (simulation study) structure to the
        associated Bill Of Process (BOP) structure. It publishes the changes from complete simulation study structure
        or the selected BOPLine objects from the simulation study structure.
        
        Use cases:
        Following use cases are supported.
        &bull;    Use Case 1: User selects one or more object(s) from simulation study structure and selects "Publish
        From Study" command. Only the selected objects are published to the associated BOP structure.
        
        &bull;    Use Case 2: User selects "Publish From Study" command without any selection, all the BOPLine nodes
        under simulation study structure are published to the associated BOP structure.
        """
        return cls.execute_soa_method(
            method_name='publishSelectionFromStudyToSource',
            library='Manufacturing',
            service_date='2019_06',
            service_name='DataManagement',
            params={'input': input},
            response_cls=SelectedStudySourceResponse,
        )

    @classmethod
    def syncSelectionInStudyWithSource(cls, input: List[SelectedSyncPublishStudyInput]) -> SelectedStudySourceResponse:
        """
        This service operation synchronizes Mfg0BvrSimStudy (simulation study) structure with the associated Bill Of
        Process (BOP) structure. It synchronizes the complete study structure or the selected BOPLine objects from the
        study.
        
        Use cases:
        Following use cases are supported. 
        &bull;    Use Case 1: User selects one or more object(s) from simulation study and selects "synchronize study"
        command . Only the selected objects are synchronized with the associated Bill Of Process (BOP) structure.
        
        &bull;    Use Case 2: User selects "synchronize study" command without any selection, all the BOPLine nodes
        under simulation study are synchronized with the associated Bill Of Process (BOP) structure.
        """
        return cls.execute_soa_method(
            method_name='syncSelectionInStudyWithSource',
            library='Manufacturing',
            service_date='2019_06',
            service_name='DataManagement',
            params={'input': input},
            response_cls=SelectedStudySourceResponse,
        )



from __future__ import annotations

from tcsoa.gen.BusinessObjects import BusinessObject
from typing import List
from tcsoa.gen.Server import ServiceData
from tcsoa.utils import TcBaseObj
from dataclasses import dataclass


@dataclass
class FileTicketDetails(TcBaseObj):
    """
    The details of the file ticket.
    
    :var fileName: The name of the log file.
    :var fileTicket: The file management server ticket of the file.
    """
    fileName: str = ''
    fileTicket: str = ''


@dataclass
class AppPathRootWindowDetails(TcBaseObj):
    """
    The apperance path root window and its type.
    
    :var productWindowAppPathRoot: The appearance path root window.
    :var appPathRootType: The type of appearance path root window.
    """
    productWindowAppPathRoot: BusinessObject = None
    appPathRootType: BusinessObject = None


@dataclass
class GenerateIPATreeResponse(TcBaseObj):
    """
    ServiceData element, appearance path root window along with its type which was generated during the operation and
    the log file details and its ticket. The newly created occurrence groups from the process strcture and appearance
    path root of the product strcuture is added to the Created object list of ServiceData. Partial errors occurred in
    the operation are added to ServiceData.
    
    :var serviceData: The Service Data.
    :var productWindowAppPathRoot: The appearance window which was created.
    :var appPathRootType: The type of appearance window which was created.
    :var logFileTicket: The details of the log file ticket.
    """
    serviceData: ServiceData = None
    productWindowAppPathRoot: BusinessObject = None
    appPathRootType: BusinessObject = None
    logFileTicket: FileTicketDetails = None


@dataclass
class IPAExistResponse(TcBaseObj):
    """
    ServiceData element, flag indicating whether IPA tree exists for the process structure and ticket of configuration
    file if the IPA tree exists for the structure. Partial errors occurred in Teamcenter internal APIs during the
    operation are returned in ServiceData.
    
    :var serviceData: The Service Data.
    :var isIPAExist: The flag indicating whether IPA tree exists for the process structure.
    :var configFileTicket: The ticket of configuration file. This file is saved as an attachment on the top-level
    process element on which the assembly tree was generated/updated. This file contains information about all the
    configured product structures consumed in the process structure.
    """
    serviceData: ServiceData = None
    isIPAExist: bool = False
    configFileTicket: FileTicketDetails = None


@dataclass
class IPATreeInput(TcBaseObj):
    """
    All the details for IPA generation that includes the name of the IPA, process types, consumption types, occurrence
    group type, effectivity, email notification details etc.
    
    :var configDetails: The configuration details about the IPA generation which includes details of process structure,
    consumption types, occurrence group types and effectivity details.
    :var emailDetails: The e-mail notification details like recipients of the e-mail, subject and content of the
    message.
    """
    configDetails: ConfigDetails = None
    emailDetails: EmailNotificationDetails = None


@dataclass
class LocalUpdateIPATreeResponse(TcBaseObj):
    """
    ServiceData element, the log file name and its ticket. The updated occurrence groups from the process structure and
    appearance path root of the product structure is added to the Updated object list of ServiceData. Partial errors
    occurred in the operation are added to ServiceData.
    
    :var serviceData: The Service Data.
    :var logFileticket: The details about the log file ticket.
    """
    serviceData: ServiceData = None
    logFileticket: FileTicketDetails = None


@dataclass
class CleanIPATreeResponse(TcBaseObj):
    """
    ServiceData element, appearance path root window along with its type which was deleted during the operation and the
    log file details and its ticket. The deleted occurrence groups from the process structure and deleted appearance
    path root of the product structure is added to the Deleted object list of ServiceData. Partial errors occurred in
    the operation are added to ServiceData.
    
    :var serviceData: The Service Data.
    :var appPathRootWindows: List of appearance windows deleted by the operation. These windows corressponds to product
    structures consumed in the process strcuture to be cleaned.
    :var logFileTicket: The details of the log file ticket.
    """
    serviceData: ServiceData = None
    appPathRootWindows: List[AppPathRootWindowDetails] = ()
    logFileTicket: FileTicketDetails = None


@dataclass
class ConfigDetails(TcBaseObj):
    """
    The configuration details about the IPA generation which includes details of process structure, consumption types,
    occurrence group types and effectivity details.
    
    :var processWindow: The window of the process structure in which items are consumed from a product structure.
    :var ipaName: The name of the IPA to be processed.
    :var occGrpType: The type of occurrence group to be generated for the assembly tree.
    :var processTypes: List of process types for which an assembly tree is generated.
    :var consumptionOccTypes: List of consumed occurrence types considered for generation of the assembly tree.
    :var effectivityDetails: The effectivity details for the IPA generation like end item, its revision, range and/or
    unit of effectivity.
    """
    processWindow: BusinessObject = None
    ipaName: str = ''
    occGrpType: str = ''
    processTypes: List[str] = ()
    consumptionOccTypes: List[str] = ()
    effectivityDetails: EffectivityDetails = None


@dataclass
class EffectivityDetails(TcBaseObj):
    """
    The effectivity details for the IPA generation like end item, its revision, range and/or unit of effectivity.
    
    :var effectivityEndItem: The end item of generated assembly tree to which the effectivity is associated.
    :var effectivityEndItemRev: The end item revision of generated assembly tree to which the effectivity is associated.
    :var effectivityUnitRange: The unit range of effectivity associated with IPA generated occurrence.
    :var effectivityDateRange: The date range of effectivity associated with IPA generated occurrence.
    """
    effectivityEndItem: BusinessObject = None
    effectivityEndItemRev: BusinessObject = None
    effectivityUnitRange: str = ''
    effectivityDateRange: str = ''


@dataclass
class EmailNotificationDetails(TcBaseObj):
    """
    The e-mail notification details like recipients of the e-mail, subject and content of the message.
    
    :var logFileName: This is name of the report on errors and problems during the operation. This report contains all
    the problems or conflicts that occurred during generate/update that need to be addressed, for example, missing
    targets, locked nodes, or checked out nodes.
    This file is saved as an attachment on the top-level process element on which the assembly tree was
    generated/updated. 
    
    :var isLogFileNeeded:  This flag is used to create a run-time statistic log for IPA generation.
    :var mailToReceivers: List of recipients to be added in 'To' list of the e-mail notification.
    :var mailCcReceivers: List of recipients to be added in 'Cc' list of the e-mail notification.
    :var mailSubject: The subject of the e-mail notification.
    :var mailMessage: The message content of the e-mail notification.
    """
    logFileName: str = ''
    isLogFileNeeded: bool = False
    mailToReceivers: List[str] = ()
    mailCcReceivers: List[str] = ()
    mailSubject: str = ''
    mailMessage: str = ''



from __future__ import annotations

from tcsoa.gen.BusinessObjects import Group, ImanAliasList, User
from typing import List
from tcsoa.gen.Server import ServiceData
from tcsoa.utils import TcBaseObj
from dataclasses import dataclass


@dataclass
class UserGroupAliasResponse(TcBaseObj):
    """
    Lists of User, Group, and ImanAliasList objects from the operation.
    
    :var users: A list of all Teamcenter User objects
    :var groups: A list of all Teamcenter Group objects
    :var aliasLists: The list of all ImanAliasList objects
    :var serviceData: Error encountered while processing post event on element in the set is reported as partial errors
    in serviceData and processing continues for the remaining elements in the input set.
    """
    users: List[User] = ()
    groups: List[Group] = ()
    aliasLists: List[ImanAliasList] = ()
    serviceData: ServiceData = None



from __future__ import annotations

from tcsoa.gen.BusinessObjects import BusinessObject
from tcsoa.gen.Core._2015_07.DataManagement import CreateInput2
from tcsoa.utils import TcBaseObj
from dataclasses import dataclass


@dataclass
class CreateIn3(TcBaseObj):
    """
    This is an input structure for create operation of a single object including unique client identifier.
    
    :var clientId: Unique client identifier.
    :var createData: Input data for create operation.
    :var targetObject: Target to which the created object will be related.
    :var pasteProp: Property to be used to relate the created object to the targetObject. This can be something similar
    to the illustrations below:
    (1)      A reference property on the targetObject Type. For example, "contents" on targetObject of Type Folder.
    (2)      A relation property on the targetObject Type. For example, "IMAN_reference" on targetObject of Type Item
    or "IMAN_specification" on targetObject of Type ItemRevision.
    """
    clientId: str = ''
    createData: CreateInput2 = None
    targetObject: BusinessObject = None
    pasteProp: str = ''



from __future__ import annotations

from tcsoa.gen.BusinessObjects import BusinessObject
from typing import List
from tcsoa.utils import TcBaseObj
from dataclasses import dataclass


@dataclass
class PreferenceDefinition(TcBaseObj):
    """
    The PreferenceDefinition structure represents the definition for a preference
    in Teamcenter.
    
    :var preferenceName: Name of the preference.
    :var preferenceCategory: Category of the preference.
    :var preferenceDescription: Description of the preference.
    :var preferenceScope: Scope of the preference. It can be "USER", "ROLE", "GROUP" or "SITE".
    :var preferenceType: Type of the preference. It can be "STRING", "LOGICAL", "INTEGER", "DOUBLE" or "DATE".
    :var isArray: Specifies if this preference takes multiple values.
    :var isDisabled: Specifies if this preference is enabled.
    """
    preferenceName: str = ''
    preferenceCategory: str = ''
    preferenceDescription: str = ''
    preferenceScope: str = ''
    preferenceType: str = ''
    isArray: bool = False
    isDisabled: bool = False


@dataclass
class PreferenceInfo(TcBaseObj):
    """
    Holds the complete information about a preference: its definition is captured in the PreferenceDefinition
    structure, and its values in the list of Context structures. 
    
    
    :var preferenceDefinition: A structure holding the definition of the preference.
    :var contextInformation: A list of Context structures, each representing the values for a given context.
    """
    preferenceDefinition: PreferenceDefinition = None
    contextInformation: List[Context] = ()


@dataclass
class PreferencesInfo(TcBaseObj):
    """
    Contains a list of PreferenceInfo structures.
    
    :var prefsInfo: List of PreferenceInfo structure.
    """
    prefsInfo: List[PreferenceInfo] = ()


@dataclass
class PreferencesSetInput(TcBaseObj):
    """
    Input to be provided for setting preferences.
    
    :var preferenceScope: Location of the preference.
    :var inputPrefs: Preferences to be set.
    :var object: Object to which the preferences have to be assigned.
    This can be a User, Role or Group. If no value is specified, preferences are set for the current session based on
    the scope. 
    To be able to set preferences for the site, the user must be a system administrator.
    To be able to set preferences for a group or a role, the user must be group administrator or a system administrator.
    """
    preferenceScope: str = ''
    inputPrefs: PreferencesInfo = None
    object: BusinessObject = None


@dataclass
class Context(TcBaseObj):
    """
    Represents the preference context values.
    
    :var contextName: Context name for the values. 
    
    The default one is "Teamcenter". It can also be set to be used with NXManager, by setting the TC_Application
    environment variable to UGMAN. 
    
    :var value: List of values applicable for the context.
    """
    contextName: str = ''
    value: List[str] = ()



from __future__ import annotations

from tcsoa.gen.BusinessObjects import BusinessObject, ReportDefinition
from typing import Dict, List
from tcsoa.gen.Server import ServiceData
from tcsoa.utils import TcBaseObj
from dataclasses import dataclass


@dataclass
class AccessorInfo(TcBaseObj):
    """
    This structure is used to pass the accessor information( combination of Group UID, Role UID and User UID ).
    Supported use cases are:
    1. Group UID is passed, Role UID and User UID are empty, which means the Awp0SearchFolder is expected to be shared
    with the particular group.
    2. Role UID and Group UID are passed, User UID is empty, which means the Awp0SearchFolder is expected to be shared
    with the particular role within the particular group.
    3. User UID is passed, Role UID and Group UID are empty, which means the Awp0SearchFolder is expected to be shared
    with the particular user.
    
    :var groupUID: Specifies the UID of the Group with which the input Awp0SearchFolder is expected to be shared with.
    :var roleUID: Specifies the UID of the Role with which the input Awp0SearchFolder is expected to be shared with.
    :var userUID: Specifies the UID of the User with which the input Awp0SearchFolder is expected to be shared with.
    """
    groupUID: str = ''
    roleUID: str = ''
    userUID: str = ''


@dataclass
class GetSearchFolderAccessorsOutput(TcBaseObj):
    """
    Output structure that holds the accessors info for provided Awp0SearchFolder UIDs.
    
    :var searchFolderAndItsAccessors: A map (string, list of business objects) to hold the Awp0SearchFolder and list of
    its associated accessors. Supported types for the business objects are POM_accessor, POM_group and POM_user or
    GroupMember.
    :var serviceData: Service data for holding the partial errors.
    """
    searchFolderAndItsAccessors: SearchFolderAndItsAccessorsMap = None
    serviceData: ServiceData = None


@dataclass
class ImportInput(TcBaseObj):
    """
    Structure defines the mode, the UID of the input Awp0SearchFolder object and the transient file ticket for the
    TCXML file specified in the Active Workspace client for the importSearchFolder service operation.
    The mode parameter has two valid values, preview and import. The preview mode is a way for the user to see what
    structure is going to be imported into the input Awp0SearchFolder object. The import mode is when the user wants to
    do the TCXML import of the Awp0SearchFolder hierarchy along with its ReportDefinition and Awp0SearchFolderShareRule
    which is specified in the TCXML into the input Awp0SearchFolder object.
    The searchFolderUID specifies the UID of the input active folder under which the hierarchy is to be imported.
    If it is passed as empty, then the import will happen into the Awp0MySearchFolder instance of the logged in user.
    
    :var mode: Specifies whether to get the preview of the Awp0SearchFolder with its hierarchy or to import the
    Awp0SearchFolder and its hierarchy. Valid values are: "import" and "preview".
    Import:  Used to import the Awp0SearchFolder object,  its children, and Awp0SearchFolderShareRule and
    ReportDefinition objects attached to it as a child of the searchFolderUID. 
    Preview:  Provides the preview of the root Awp0SearchFolder object specified in the TCXML along with its hierarchy
    (its children) in tree view mode in a JSON string format.
    :var transientFileTicket: The file ticket for the TCXML to be imported into the selected Awp0SearchFolder.
    (required)
    :var searchFolderUID: UID of the input Awp0SearchFolder under which the contents of the TCXML needs to be imported.
    Required for import mode; otherwise, it is empty.
    """
    mode: str = ''
    transientFileTicket: str = ''
    searchFolderUID: str = ''


@dataclass
class ImportSearchFolderResponse(TcBaseObj):
    """
    The response contains the serviceData and the preview string in JSON format. The serviceData contains any partial
    errors returned from the import operation using TIE import and the preview of the hierarchy of the Awp0SearchFolder
    object specified in the input TCXML file. Hierarchy meaning it will mention the name of the root Awp0SearchFolder
    and the names of its children active folders till the leaf Awp0SearchFolder object. The previewStringJSON will be
    only returned if the input mode parameter is passed as preview. Otherwise for the import mode, it will be empty.
    
    Structure of JSON schema:
    
    "dataForTheTree":
    [ 
        { 
              "label": "Active Folder 001", 
              "children": 
               [
                   { 
                        "label": "Active Folder 001-Child", 
                        "children":
                        [
                             {
                                    "label": "Active Folder Sub Child", 
                                 "children":[]
                             }
                        ]
                   }, 
                  {
                         "label": "Active Folder 002-Child"
                   }
                  ] 
        }
    ]
    
    :var serviceData: Service data for holding the partial errors returned for the import of the transient file ticket
    or preview of the TCXML file to be imported.
    :var previewStringJSON: A preview of the hierarchy of the Awp0SearchFolder specified in the TCXML. Output is in
    JSON format which the client will read to show the hierarchy in tree view mode. Only returned when the mode is
    preview. Empty for the import mode.
    """
    serviceData: ServiceData = None
    previewStringJSON: str = ''


@dataclass
class ReportChartObject(TcBaseObj):
    """
    Teamcenter type and property name for which charting information is desired.
    
    :var chartTypeName: Internal name of the Teamcenter type.
    :var chartPropertyName: Internal name of the Teamcenter property.
    :var isPropertyLocalized: If true, indicates that localized value of this property has been indexed in the search
    engine.
    :var chartPropertyType: Teamcenter property type.
    """
    chartTypeName: str = ''
    chartPropertyName: str = ''
    isPropertyLocalized: bool = False
    chartPropertyType: str = ''


@dataclass
class ReportSearchRecipe(TcBaseObj):
    """
    Structure to represent Search recipe for individual Teamcenter ReportDefinition object.
    
    :var reportObject: Teamcenter ReportDefinition object.
    :var translatedBaseCriteria: Translated primary search criteria.
    :var translatedFilterQueries: An list of translated filter criteria.
    :var reportChartObjects: A list of chart objects supported by the ReportDefinition object.
    :var reportSearchRecipeExtraInfo: A map (string, string) of additional search related parameter and value pairs.
    Following parameters are supported: lastTotalFound: Total number of objects returned when this ReportDefinition was
    last executed by the current user.
    lastExecutedTime: Timestamp of when this ReportDefinition object was last executed by the current user.
    """
    reportObject: ReportDefinition = None
    translatedBaseCriteria: str = ''
    translatedFilterQueries: List[str] = ()
    reportChartObjects: List[ReportChartObject] = ()
    reportSearchRecipeExtraInfo: SearchRecipeExtraInfo = None


@dataclass
class ReportSearchRecipeInput(TcBaseObj):
    """
    Input criteria to retrieve ReportDefinition objects. One of reportID or reportUID is mandatory.
    
    :var reportID: The unique ID of the Teamcenter ReportDefinition object to be retrieved (optional if reportUID is
    provided).
    :var reportUID: The UID of the ReportDefinition object (optional if reportID is provided).
    :var reportSource: Data source type for the ReportDefinition object. Example: Active Workspace, Teamcenter
    (optional).
    """
    reportID: str = ''
    reportUID: str = ''
    reportSource: str = ''


@dataclass
class ReportSearchRecipeResponse(TcBaseObj):
    """
    Structure to hold a list of ReportSearchRecipe structures and search filter queries that are common for the entire
    session.
    
    :var reportSearchRecipeObjects: A list of ReportSearchRecipe structures. Each ReportSearchRecipe holds search
    recipe specific to a single ReportDefinition object.
    :var commonSearchParameters: Search filters that are common for the entire session. This includes:
    globalRevisionRule: Search filter query representing the active global revision rule.
    userContextInfo : Search filter query representing the current session context info.
    additionalFilters : Other search filter queries that are common for the entire session.
    :var serviceData: Teamcenter service data containing ReportDefinition objects being returned.
    """
    reportSearchRecipeObjects: List[ReportSearchRecipe] = ()
    commonSearchParameters: SearchRecipeExtraInfo = None
    serviceData: ServiceData = None


@dataclass
class SearchFoldersInput(TcBaseObj):
    """
    Input structure to create or edit the Awp0SearchFolder passed in the searchFolderUID.
    
    :var parentFolderUID: Parent Folder UID for the Awp0SearchFolder being created. Set to empty in edit case.
    :var searchFolderUID: UID of the Awp0SearchFolder which is being edited. Set to empty in creation case.
    :var reportDefinitionUID: UID of the ReportDefinition object that is being edited. Only passed in case of editing
    the search definition. Otherwise empty.
    :var searchCriteria: A list that contains different search criteria and their values.
    
    Example:
    
    If user is doing Active Workspace global search and wants to search for all items in the system that match the
    keyword "hdd" and is owned by user "ed", in this case the searchCriteria list would be as follows: 
    CriteriaAndValue[0] {    criteriaName "searchString"
    criteriaValues ["hdd"] }
    CriteriaAndValue[1]{    criteriaName "owning_user"
    criteriaValues ["ed"] }
    :var searchFolderAttributes: Map (string, list of strings) of Awp0SearchFolder attribute names and their values.
    The following attributes can be set when an Active Folder is created.
        Name,
        Description,
        Shared with User,
        Shared with Role,
        Shared with Group,
        Shared with Project.
                            
    Example:     
    Creation Case:
    When "Active Folder 1" is created and shared with Group1, Group2 and with Project1, entries of this map are going
    to be following: 
    Name    {Specification Search}
    Description                {All active 
                specifications}
    Shared with Group        {Group1UID, Group2UID}
    Shared with Project       {Project1UID}
    
    An Activer Folder with name "Specification Search" and description "All active specifications" is created and
    shared with Group1, Group2 and Project1.
    
    Edit Case:
    If "Specification Search" folder needs to be shared with User1 and User2, now entries of this map are going to be
    following: 
    Shared with User    {User1UID, User2UID}
    
    Now Active Folder is also shared with User1 and User2.
    :var searchFolderAccessors: A list used to pass the accessor information( combination of Group UID, Role UID and
    User UID ).
    Supported use cases are:
    1. Group UID is passed, Role UID and User UID are empty, which means the Awp0SearchFolder is expected to be shared
    with the particular group.
    2. Role UID and Group UID are passed, User UID is empty, which means the Awp0SearchFolder is expected to be shared
    with the particular role within the particular group.
    3. User UID is passed, Role UID and Group UID are empty, which means the Awp0SearchFolder is expected to be shared
    with the particular user.
    """
    parentFolderUID: str = ''
    searchFolderUID: str = ''
    reportDefinitionUID: str = ''
    searchCriteria: List[CriteriaAndValue] = ()
    searchFolderAttributes: SearchFolderAttributes = None
    searchFolderAccessors: List[AccessorInfo] = ()


@dataclass
class CreateOrEditSearchFoldersOutput(TcBaseObj):
    """
    Structure holds the createOrEditSearchFolders operation response.
    
    :var searchFolders: A list of created or edited instances of Awp0SearchFolder along with ReportDefinition objects.
    :var serviceData: Service data for holding the partial errors.
    """
    searchFolders: List[CreatedOrEditedSearchFolder] = ()
    serviceData: ServiceData = None


@dataclass
class CreatedOrEditedSearchFolder(TcBaseObj):
    """
    Structure holds the created or edited Awp0SearchFolder and ReportDefinition objects.
    
    :var searchFolder: Created or edited Awp0SearchFolder business object.
    :var reportDefinition: The created or edited ReportDefinition business object.
    :var searchFolderAttributes: Search Folder attributes map (string, a list strings).
    """
    searchFolder: BusinessObject = None
    reportDefinition: BusinessObject = None
    searchFolderAttributes: SearchFolderAttributes = None


@dataclass
class CriteriaAndValue(TcBaseObj):
    """
    This structure holds the criteria name and its values for representing a search definition.It has 2 special cases,
    for FullTextSearch case searchString will be sent as key search string. And in the case of Advanced Search,
    savedQueryUID will be sent as key to identify associated SavedQuery.
    
    :var criteriaName: Search criteria name. Supported values are: "searchString" in case of FullTextSearch.
    "savedQueryUID" in case of Advanced Search. These are 2 known values for this attribute rest of the values are
    dynamic. For example, for a given search, user can provide any filter to limit the search results. Example of
    filter values can be project list, last modified date or release status.
    :var criteriaValues: A list of the criteria values. Supported values: ImanQuery UID value in case of SavedSearch.
    Keyword search value in case of FullTextSearch. Filter values for any filter criteria. 
    Example: For finding all items matching criteria hdd for owning user ed, CriteriaAndValue is going to be
    CriteriaAndValue[0] {criteriaName searchString, criteriaValues [hdd ]}CriteriaAndValue[1]{criteriaName
    owning_user,criteriaValues [ed] }
    """
    criteriaName: str = ''
    criteriaValues: List[str] = ()


@dataclass
class ExportSearchFoldersResponse(TcBaseObj):
    """
    Structure holds the file tickets of the exported TCXML files.
    
    :var transientFileReadTickets: A list of transient file read tickets for the exported files.
    :var serviceData: Service data for holding the partial errors while generating file tickets.
    """
    transientFileReadTickets: List[str] = ()
    serviceData: ServiceData = None


"""
This map holds search folder and list of the business objects which are its accssors. Key is Awp0SearchFolder UID and the corresponding value is a vector of business objects( supported types are POM_accessor, GroupMember, POM_group and POM_user ) containing the accessors.
"""
SearchFolderAndItsAccessorsMap = Dict[str, List[BusinessObject]]


"""
Map (string, list of strings) of Awp0SearchFolder attribute names and their values.
The following attributes can be set when a Awp0SearchFolder is created or edited.
    Name,
    Description,
    Shared with User,
    Shared with Role,
    Shared with Group,
    Shared with Project.
                        
Example:     
Creation Case:
When "Active Folder1" is created and shared with Group1, Group2 and with Project1, entries of this map are going to be following: 
Name    {Specification Search}
Description                {All active 
            specifications}
Shared with Group        {Group1UID, Group2UID}
Shared with Project       {Project1UID}

An Active Folder with name "Specification Search" and description "All active specifications" is created and shared with Group1, Group2 and Project1.

Edit Case:
If "Specification Search" folder needs to be shared with User1 and User2, now entries of this map are going to be following: 
Shared with User    {User1UID, User2UID}
Now Active Folder is also shared with User1 and User2.
"""
SearchFolderAttributes = Dict[str, List[str]]


"""
A map (string, string) of additional search parameter and value pairs. This includes additional information sent as part of the translated Search recipe response for ReportDefinition objects.
"""
SearchRecipeExtraInfo = Dict[str, str]



from __future__ import annotations

from tcsoa.gen.BusinessObjects import BusinessObject, WorkspaceObject
from tcsoa.gen.Core._2007_01.ManagedRelations import TraceabilityInfoInput, ModifyManagedRelationInput, ManagedRelationResponse, CreateManagedRelationInput, TraceabilityReportOutput
from tcsoa.gen.Core._2007_01.DataManagement import SaveAsNewItemInfo, GetDatasetCreationRelatedInfoResponse, FormInfo, WhereReferencedResponse, WhereUsedResponse, GetItemFromIdPref, GenerateUIDResponse, MoveToNewFolderResponse, NameValueMap, GetItemCreationRelatedInfoResponse, CreateOrUpdateFormsResponse, GetItemFromIdResponse, GetItemFromIdInfo, SaveAsNewItemResponse, MoveToNewFolderInfo
from tcsoa.gen.Core._2007_01.Session import GetTCSessionInfoResponse, ScopedPreferenceNames, MultiPreferencesResponse
from tcsoa.gen.Core._2007_01.FileManagement import GetTransientFileTicketsResponse, TransientFileInfo
from typing import List
from tcsoa.gen.Server import ServiceData
from tcsoa.utils import TcService


class SessionService(TcService):

    @classmethod
    def getTCSessionInfo(cls) -> GetTCSessionInfoResponse:
        """
        This operation gets information about the current user's Teamcenter session. This will return more detail
        session information than the login service operation including User, Group, Role, Site, Volume, Project, and
        WorkContext.
        """
        return cls.execute_soa_method(
            method_name='getTCSessionInfo',
            library='Core',
            service_date='2007_01',
            service_name='Session',
            params={},
            response_cls=GetTCSessionInfoResponse,
        )

    @classmethod
    def setObjectPropertyPolicy(cls, policyName: str) -> bool:
        """
        Sets the current object property policy. The business logic of a service operation determines what business
        objects are returned, while the object property policy determines which properties are returned on each
        business object instance. This allows the client application to determine how much or how little data is
        returned based on how the client application uses those returned business objects. The policy is applied
        uniformly to all service operations. 
        By default, all applications use the Default object property policy, defined on the Teamcenter server
        '$TC_DATA/soa/policies/default.xml. 'It is this policy that is applied to all service operation responses until
        the client application changes the policy. Siemens PLM Software strongly recommends that all applications
        change the policy to one applicable to the client early in the session.
        The object property policy is set to the policy named in the file '$TC_DATA/soa/policies/<policyName>.xml 'The
        reserved policy name "Empty", will enforce a policy that only returns minimum data required for each object
        (UID and type name).The object property policy will stay in affect for this session until changed by another
        call to 'setObjectPRopertyPolicy'.
        
        
        Like any other service operation, this operation cannot be called before establishing a session with the
        'login' serivce operation, so if you need a policy other than the Default policy for the business objects
        returned by the 'login' operation, use the _2011_06 version of the 'login/loginSso' operation to authenticate
        and establish a session without returning business objects. The 'setObjectPropertyPolicy' operation can then be
        called to establish the policy for the session.
        
        Exceptions:
        >If the named policy does not exist or there are errors parsing the XML file (error code 214104).
        """
        return cls.execute_soa_method(
            method_name='setObjectPropertyPolicy',
            library='Core',
            service_date='2007_01',
            service_name='Session',
            params={'policyName': policyName},
            response_cls=bool,
        )

    @classmethod
    def getPreferences(cls, requestedPrefs: List[ScopedPreferenceNames]) -> MultiPreferencesResponse:
        """
        Get preference values
        
        Exceptions:
        >None
        """
        return cls.execute_soa_method(
            method_name='getPreferences',
            library='Core',
            service_date='2007_01',
            service_name='Session',
            params={'requestedPrefs': requestedPrefs},
            response_cls=MultiPreferencesResponse,
        )


class ManagedRelationsService(TcService):

    @classmethod
    def getTraceReport(cls, input: TraceabilityInfoInput) -> TraceabilityReportOutput:
        """
        This operation will create traceability report for the selected TC object.
        """
        return cls.execute_soa_method(
            method_name='getTraceReport',
            library='Core',
            service_date='2007_01',
            service_name='ManagedRelations',
            params={'input': input},
            response_cls=TraceabilityReportOutput,
        )

    @classmethod
    def modifyRelation(cls, newInput: List[ModifyManagedRelationInput]) -> ManagedRelationResponse:
        """
        This operation will Edit the managed relation
        """
        return cls.execute_soa_method(
            method_name='modifyRelation',
            library='Core',
            service_date='2007_01',
            service_name='ManagedRelations',
            params={'newInput': newInput},
            response_cls=ManagedRelationResponse,
        )

    @classmethod
    def createRelation(cls, relationinfo: List[CreateManagedRelationInput]) -> ManagedRelationResponse:
        """
        This operation will create new managed relation
        """
        return cls.execute_soa_method(
            method_name='createRelation',
            library='Core',
            service_date='2007_01',
            service_name='ManagedRelations',
            params={'relationinfo': relationinfo},
            response_cls=ManagedRelationResponse,
        )


class FileManagementService(TcService):

    @classmethod
    def getTransientFileTicketsForUpload(cls, transientFileInfos: List[TransientFileInfo]) -> GetTransientFileTicketsResponse:
        """
        This operation gets the tickets for the desired files to be uploaded to the transient volume. These tickets can
        be used to upload corresponding files via 'FileManagementUtility::putFileViaTicket'. The 'TransientFileInfo'
        contains the basic information for a file to be uploaded such as file name, file type and whether the file
        should be deleted after reading.
        
        Use cases:
        This operation supports the uploading of files into the FMS transient volume.
        """
        return cls.execute_soa_method(
            method_name='getTransientFileTicketsForUpload',
            library='Core',
            service_date='2007_01',
            service_name='FileManagement',
            params={'transientFileInfos': transientFileInfos},
            response_cls=GetTransientFileTicketsResponse,
        )


class DataManagementService(TcService):

    @classmethod
    def moveToNewFolder(cls, moveToNewFolderInfos: List[MoveToNewFolderInfo]) -> MoveToNewFolderResponse:
        """
        The 'moveToNewFolder' operation moves a set of objects from one folder to another. This operation allows for
        moving multiple sets of objects to and from different folders. If no old folder is specified, this operation
        adds the objects to the new folder.
        
        
        Use cases:
        - The user selects an object or group of objects and specifies the folder for the objects to be copied into.
        - The user selects an object or group of objects, removes them from a specified folder and specifies the folder
        for the objects to be copied into.
        
        """
        return cls.execute_soa_method(
            method_name='moveToNewFolder',
            library='Core',
            service_date='2007_01',
            service_name='DataManagement',
            params={'moveToNewFolderInfos': moveToNewFolderInfos},
            response_cls=MoveToNewFolderResponse,
        )

    @classmethod
    def refreshObjects(cls, objects: List[BusinessObject]) -> ServiceData:
        """
        This operation is used to reload the in-memory representation of the objects from the database. Any references
        to the object will still be valid. Any in-memory changes to the original object will be lost. If the object has
        been changed in the database since it was last loaded, then those changes will not be present in memory. The
        operation updates the in memory representation to reflect database changes and does not obtain write lock on
        any objects.
        
        Use cases:
        Use this operation to reload the in-memory representation of one or more objects from the Teamcenter database.
        """
        return cls.execute_soa_method(
            method_name='refreshObjects',
            library='Core',
            service_date='2007_01',
            service_name='DataManagement',
            params={'objects': objects},
            response_cls=ServiceData,
        )

    @classmethod
    def saveAsNewItem(cls, info: List[SaveAsNewItemInfo]) -> SaveAsNewItemResponse:
        """
        This operation creates a new Item object and ItemRevision object from an existing ItemRevision object.  The
        master form properties may be supplied for the new ItemRevision and item master form objects.  If master form
        data is not supplied the master forms will be initialized from the master forms attached to the existing
        ItemRevision.  Deep Copy rules may also be supplied to override the default Deep Copy rules.
        """
        return cls.execute_soa_method(
            method_name='saveAsNewItem',
            library='Core',
            service_date='2007_01',
            service_name='DataManagement',
            params={'info': info},
            response_cls=SaveAsNewItemResponse,
        )

    @classmethod
    def setProperties(cls, objects: List[BusinessObject], attributes: NameValueMap) -> ServiceData:
        """
        This operation is to support updating list of objects with the given property names and values. All the objects
        updated with same set of property and values data.  Also see  Teamcenter::Soa::Core::2010_09::setProperties
        operation.
        Note: Objects are saved as a part of this operation itself.
        """
        return cls.execute_soa_method(
            method_name='setProperties',
            library='Core',
            service_date='2007_01',
            service_name='DataManagement',
            params={'objects': objects, 'attributes': attributes},
            response_cls=ServiceData,
        )

    @classmethod
    def whereReferenced(cls, objects: List[WorkspaceObject], numLevels: int) -> WhereReferencedResponse:
        """
        This operation finds the objects and relations that reference a given object.  It returns objects where the
        input object is specified in a Reference property on that object.  It also returns relations where the input
        object is listed as the secondary object for that relation.  It does not return relations where the input
        object is the primary object for that relation. The Datamanagement service operation
        expandGRMRelationsForPrimary can be used to return the relations where the input object is the primary object
        and the objects which are the secondary object for the relation.
        
        Use cases:
        User selects an object, specifies the number of levels (or all) of referencers to return and executes a where
        referenced query.
        
        For example, the user selects a Dataset which has a specification relation to an Item and is contained in the
        users Home folder. The Item is contained in the user Newstuff folder and in the view folder of another Item
        Revision. If the user selects level 2, the Item and Home folder would be returned at level 1 and the Newstuff
        folder and view folder of the other ItemRevision would be returned at level 2.
        """
        return cls.execute_soa_method(
            method_name='whereReferenced',
            library='Core',
            service_date='2007_01',
            service_name='DataManagement',
            params={'objects': objects, 'numLevels': numLevels},
            response_cls=WhereReferencedResponse,
        )

    @classmethod
    def whereUsed(cls, objects: List[BusinessObject], numLevels: int, whereUsedPrecise: bool, rule: BusinessObject) -> WhereUsedResponse:
        """
        The 'whereUsed' service identifies all the parent Item and ItemRevision objects in the structure where the
        input Item or ItemRevision is used. User can provide RevisionRule to search for specific ItemRevision. By
        default all ItemRevision objects are returned. The number of levels of 'whereUsed' search indicates, whether to
        return one or top or all levels of assemblies. It supports search on Item, ItemRevision  and Dataset.
        
        Use cases:
        A user performs 'whereUsed' search to find all the assemblies that contain a particular Item or ItemRevision.
        User inputs Item or ItemRevision and the search can be made with following options:
        - RevisionRule This can be set to All, displaying all ItemRevision objects  that have an occurrence of target
        ItemRevision. If a specific RevisionRule is selected only the ItemRevision objects  configured by the rule are
        returned in the search.
        - Depth up to which numbers of levels are to be returned.
        
        
        
        The output contains list of  each parent level search result in the structure.
        """
        return cls.execute_soa_method(
            method_name='whereUsed',
            library='Core',
            service_date='2007_01',
            service_name='DataManagement',
            params={'objects': objects, 'numLevels': numLevels, 'whereUsedPrecise': whereUsedPrecise, 'rule': rule},
            response_cls=WhereUsedResponse,
        )

    @classmethod
    def createOrUpdateForms(cls, info: List[FormInfo]) -> CreateOrUpdateFormsResponse:
        """
        This operation creates Form objects or update existing Form objects using the info provided. A new Form will be
        associated to the container object with specified relation type. The properties of the existing Form will be
        updated.
        """
        return cls.execute_soa_method(
            method_name='createOrUpdateForms',
            library='Core',
            service_date='2007_01',
            service_name='DataManagement',
            params={'info': info},
            response_cls=CreateOrUpdateFormsResponse,
        )

    @classmethod
    def generateUID(cls, nUID: int) -> GenerateUIDResponse:
        """
        This function returns a number of Teamcenter UIDs generated from the Teamcenter server. This operation can be
        used for assigning unique identifiers to objects that will not be stored in Teamcenter or for objects which
        have yet to be created in Teamcenter.
        
        The 'createObjects' and 'createOrUpdateParts' operations will support input of a preallocated UID for use
        during creation. Please see those operation descriptions for further details.
        
        
        Use cases:
        The integration create workflow requires data to be precreated and stored outside of Teamcenter and then used
        during the Teamcenter create process. For example, generating a UID for an ItemRevision object and then storing
        that UID in the CAD integration data file. The UID is then used as input to the create SOA operation and that
        UID is assigned to the created object.
        """
        return cls.execute_soa_method(
            method_name='generateUID',
            library='Core',
            service_date='2007_01',
            service_name='DataManagement',
            params={'nUID': nUID},
            response_cls=GenerateUIDResponse,
        )

    @classmethod
    def getDatasetCreationRelatedInfo(cls, typeName: str, parentObject: BusinessObject) -> GetDatasetCreationRelatedInfoResponse:
        """
        This operation pre-populates Dataset creation information, default new Dataset name and Tool names, for a
        specified Dataset type.  This operation is used to get all the information associates with the specified
        Dataset prior to the creation operation. The returned default new Dataset name may be determined by the parent
        container object.
        """
        return cls.execute_soa_method(
            method_name='getDatasetCreationRelatedInfo',
            library='Core',
            service_date='2007_01',
            service_name='DataManagement',
            params={'typeName': typeName, 'parentObject': parentObject},
            response_cls=GetDatasetCreationRelatedInfoResponse,
        )

    @classmethod
    def getItemCreationRelatedInfo(cls, typeName: str, parentObject: BusinessObject) -> GetItemCreationRelatedInfoResponse:
        """
        This operation will return naming rules, property rule, form property descriptor, unit of measurement and
        ItemRevision type name based on Item type selected by user during Item creation.
        """
        return cls.execute_soa_method(
            method_name='getItemCreationRelatedInfo',
            library='Core',
            service_date='2007_01',
            service_name='DataManagement',
            params={'typeName': typeName, 'parentObject': parentObject},
            response_cls=GetItemCreationRelatedInfoResponse,
        )

    @classmethod
    def getItemFromId(cls, infos: List[GetItemFromIdInfo], nRev: int, pref: GetItemFromIdPref) -> GetItemFromIdResponse:
        """
        This operation returns Items, Item Revisions, and Dataset based on the input item id. Input is a list of
        GetItemFromIdInfo structures each of which contain an item id (GetItemFromIdInfo.itemId) and optionally a list
        of revision ids (GetItemFromIdInfo.revIds) which specify which Item Revisions to retrieve.  Also input is and
        integer value (nRev) which can also be used to help specify which Item Revisions to return with the Item.  The
        final input is a GetItemFromIdPref structure which contains a list of RelationFilter structures
        (GetItemFromIdPref.prefs) each of which contain a relation type name (RelationFilter.relationTypeName) and a
        list of object type names (RelationFilter.objectTypeNames). This filter can be used to specify which Datasets
        are returned.  The relation type name specifies the relation that relates the Item Revision to the Dataset. 
        The object type name is the type of Dataset to return.  For example, if relationTypeName is "IMAN_reference"
        and the object type name is "Text" then only those Datasets of type "Text" that are related to candidate Item
        Revisions with the "IMAN_reference" relation will be returned.  Supplying no value or an empty value for the
        rev id list and 0 for nRevs will signify the return of no Item Revisions, and thus no Datasets will be returned
        either.  Supplying no value or an empty value for the rev id list and a negative value for nRevs will signify
        the return of all Item Revisions.   Supplying no value or an empty value for the rev id list and a positive
        value for nRev will signify the return of the latest number of Item Revisions specified by the integer--if the
        number of actual revisions found is greater than the input nRev, all revisions for the found Item will be
        returned. Supplying a rev id list will only return those revisions, and the nRev value will not be processed.
        For example, if the input rev Id is "A" and the nRev value is 0, only revision "A" will be returned. If the rev
        id list is empty and nRevs = 5, then the 5 latest Item Revisions will be returned. If no preference filter is
        specified, all Datasets will be returned.  The return is a GetItemFromIdResponse which contains a list of
        GetItemFromIdItemOutput (GetItemFromIdResponse.output and a ServiceData (GetItemFromIdResponse.serviceData). 
        Each GetItemFromIdItemOutput contains an Item (GetItemFromIdItemOutput.item) and a list of
        GetItemFromIdItemRevOutput structures (GetItemFromIdItemOutput.itemRevOutput).  Each GetItemFromIdItemRevOutput
        structure contains an Item Revision (GetItemFromIdItemRevOutput.itemRevision) and a list of found Datasets
        (GetItemFromIdItemRevOutput.datasets).
        """
        return cls.execute_soa_method(
            method_name='getItemFromId',
            library='Core',
            service_date='2007_01',
            service_name='DataManagement',
            params={'infos': infos, 'nRev': nRev, 'pref': pref},
            response_cls=GetItemFromIdResponse,
        )



from __future__ import annotations

from tcsoa.gen.Internal.AWS2._2017_06.Finder import ColumnConfig2
from typing import Dict, List
from tcsoa.gen.Internal.AWS2._2016_03.Finder import SearchSortCriteria, SearchFilterField, ObjectsGroupedByProperty
from tcsoa.gen.Server import ServiceData
from tcsoa.utils import TcBaseObj
from dataclasses import dataclass
from tcsoa.gen.Internal.AWS2._2017_12.Finder import SearchFilter4


@dataclass
class SearchCursor(TcBaseObj):
    """
    A structure containing cursor
    
    :var startIndex: Indicates the Cursor start position for the search result returned so far.
    :var endIndex: Indicates the Cursor end position for the search result returned so far
    :var startReached: If true, the first page of the results has been reached.
    :var endReached: If true, the last page of the results has been reached.
    """
    startIndex: int = 0
    endIndex: int = 0
    startReached: bool = False
    endReached: bool = False


@dataclass
class SearchInput2(TcBaseObj):
    """
    A structure containing input search criteria.
    
    :var providerName: The name of the search provider. This is the RuntimeBusinessObject type name.
    :var searchCriteria: A map (string, string) used to perform the search. For full text search, the key is
    "searchString", the value is "*". For advanced search, the key can be "queryID",  "quickSearchAttributeValue",
    "searchID", "typeOfSearch", "QUICK_SEARCH" or "ADVANCED_SEARCH", the value is the unique identifier.
    :var attributesToInflate: A list of attributes to inflate (extract the details).
    :var internalPropertyName: The internal name of the property used for grouping.
    :var cursor: Cursor object used in the search
    :var focusObjUid: Uid of the Object to focus to
    :var pagingType: The type of paging operation. Valid values are GetCurrentPage, GetNextPage, GetPreviousPage
    :var maxToReturn: The maximum number of search results to return. If zero, no search results are returned; however,
    other information such as total results found and filters details are returned.
    :var maxToLoad: The maximum number of search results to load.
    :var searchFilterMap: A map (string, list of SearchFilter) containing the list of search filters for each search
    filter field. The key in the map is the property name that represents the filter category. It is in the format
    "TypeName.PropertyName". e.g WorkspaceObject.release_statuses, the value in the map is the value of  the search
    filter.
    :var searchSortCriteria: The criteria to use to sort the results.
    :var searchFilterFieldSortType: The sorting type to use to order the search filter categories in the response.
    Supported values are: "Alphabetical", "Priority".
    """
    providerName: str = ''
    searchCriteria: StringMap2 = None
    attributesToInflate: List[str] = ()
    internalPropertyName: str = ''
    cursor: SearchCursor = None
    focusObjUid: str = ''
    pagingType: str = ''
    maxToReturn: int = 0
    maxToLoad: int = 0
    searchFilterMap: SearchFilterMap5 = None
    searchSortCriteria: List[SearchSortCriteria] = ()
    searchFilterFieldSortType: str = ''


@dataclass
class SearchResponse5(TcBaseObj):
    """
    A service response structure containing search results and column configurations.
    
    :var searchResultsJSON: Search results in JSON Format. With this, Client can directly bind the view model
    properties JSON to the view.
    :var totalFound: Total number of view model business objects found.
    :var cursor: Cursor for the results returned so far.
    :var searchFilterCategoriesUnpopulated: A list of empty search filter categories.
    :var totalLoaded: Total number of view model business objects loaded.
    :var searchFilterMap5: A map (string, list of Awp0::Soa::Internal::AWS2::_2017_12::Finder::SearchFilter34)
    containing the list of search filters for each search filter field based on the search results.
    :var searchFilterCategories: A list of search filter categories ordered by filter priority.
    :var defaultFilterFieldDisplayCount: The default number of search filter categories to display.
    :var objectsGroupedByProperty: The The structure, Awp0::Soa::Internal::AWS2::_2016_03::ObjectPropertyGroupingMap
    containing an internal property name and a map of objects to the property group id. It also contains a list of
    unmatched objects which do not match any group.
    :var columnConfig: Effective column configuration for the client scope URI.
    :var serviceData: The service data object.
    :var additionalSearchInfoMap: A generic map (string/list of strings) that can be used to send additional search
    information to the consumer. 
    
    Supported key values are:
    searchTermsToHighlight - used to indicate which Search Terms need to be highlighted
    """
    searchResultsJSON: str = ''
    totalFound: int = 0
    cursor: SearchCursor = None
    searchFilterCategoriesUnpopulated: List[SearchFilterField] = ()
    totalLoaded: int = 0
    searchFilterMap5: SearchFilterMap5 = None
    searchFilterCategories: List[SearchFilterField] = ()
    defaultFilterFieldDisplayCount: int = 0
    objectsGroupedByProperty: ObjectsGroupedByProperty = None
    columnConfig: ColumnConfig2 = None
    serviceData: ServiceData = None
    additionalSearchInfoMap: StringVectorMap = None


@dataclass
class FacetSearchInput(TcBaseObj):
    """
    The FacetSearchInput is based on existing structure SearchInput of performSearchViewModel3. It contains search
    criteria and search filter information.
    
    :var providerName: The name of the search provider. This is the RuntimeBusinessObject type name.
    :var searchCriteria: A map (string, string) used to perform the search. 
    For full text search, the key is "searchString", the value is input text. For example, ("searchString" : "*" | "?"
    | "string"). 
    
    For advanced search, the supported key/values are: 
    ("typeOfSearch" : "QUICK_SEARCH" or "ADVANCED_SEARCH");
    ("queryUID" :  UID to a WorkspaceObject);
    ("searchID" :  UID to a WorkspaceObject);
    ("quickSearchAttributeValue" : search string).
    
    For example, an advance search might be:
    "typeOfSearch" : "QUICK_SEARCH";
    "queryUID" :  "UID";
    "searchID" : "UID";
    "quickSearchAttributeValue": "*"
    :var searchFilterMap: A map (string, list of SearchFilter) containing the list of search filters for each search
    filter field. The key in the map is the property name that represents the filter category. It is in the format
    "TypeName.PropertyName". For example: WorkspaceObject.object_type, the value in the map is the value of the search
    filter.
    :var startIndex: The start index to return the facet values. This value must be zero or positive. Error will be
    returned for a negative value.
    :var maxToReturn: The maximum number of facet values to return  This value must be positive i.e. greater than 0,
    else error will be returned.
    """
    providerName: str = ''
    searchCriteria: StringMap2 = None
    searchFilterMap: SearchFilterMap5 = None
    startIndex: int = 0
    maxToReturn: int = 0


@dataclass
class FacetSearchResponse(TcBaseObj):
    """
    A response structure containing facet values for the filter category in input searchFilterMap.
    
    :var searchFilterMap: A map (string, list of SearchFilter) containing the list of search filters for each search
    filter field. The key in the map is the property name that represents the filter category. It is in the format
    "TypeName.PropertyName". For example: WorkspaceObject.object_type, the value in the map is the value of the search
    filter.
    :var hasMoreFacetValues: If true, client can make subsequent call for next set of facet values.
    :var endIndex: Cursor end position for the facet values returned so far. This value, 'endIndex' will be used as
    'startIndex' in FacetSearchInput for subsequent call.
    :var serviceData: The service data object.
    """
    searchFilterMap: SearchFilterMap5 = None
    hasMoreFacetValues: bool = False
    endIndex: int = 0
    serviceData: ServiceData = None


"""
A map containing a list of search filters for each search filter field.
"""
SearchFilterMap5 = Dict[str, List[SearchFilter4]]


"""
Map from string to string.
"""
StringMap2 = Dict[str, str]


"""
A map of string to list of strings
"""
StringVectorMap = Dict[str, List[str]]



from __future__ import annotations

from tcsoa.gen.BusinessObjects import BusinessObject
from typing import Dict, List
from tcsoa.gen.Internal.Core._2017_11.LogicalObject import PresentedPropertyDefinition
from tcsoa.utils import TcBaseObj
from dataclasses import dataclass


@dataclass
class LogicalObjectTypeInput2(TcBaseObj):
    """
    Holds the logical object type definition data for the specified logical object type to be created.
    
    :var name: Name of the logical object type. (user supplied name)
    :var displayName: Display name of the logical object type.
    :var description: Description of the logical object type.
    :var rootTypeName: Name of the root business object, All subtypes of POM_object are supported.
    :var parentTypeName: Name of the parent logical object type, It should be a type or subtype of Fnd0LogicalObject.
    (optional)
    :var retrieveClassificationData: If TRUE , classification data will be retrieved for the root business object.
    If FALSE , classification data will not be retrieved for the root business object.
    :var memberPropertyDefinitions: A list of member property defintions. (optional)
    :var presentedPropertyDefinitions: A list of presented property defintions. (optional)
    """
    name: str = ''
    displayName: str = ''
    description: str = ''
    rootTypeName: str = ''
    parentTypeName: str = ''
    retrieveClassificationData: bool = False
    memberPropertyDefinitions: List[MemberPropertyDefinition2] = ()
    presentedPropertyDefinitions: List[PresentedPropertyDefinition] = ()


@dataclass
class MemberPropertyDefinition2(TcBaseObj):
    """
    Member properties represents the objects, which is reached from the root business object via the specified
    traversal paths, in order to present their properties onto the logical object.
    
    :var memberPropertyName: Name of the member property.
    :var displayName: Display name of the member property.
    :var description: Description of the member property.
    :var traversalPath: A list of traversal hops.
    :var retrieveClassificationData: If TRUE , classification data will be retrieved for this member property.
    If FALSE , classification data will not be retrieved for this member property.
    """
    memberPropertyName: str = ''
    displayName: str = ''
    description: str = ''
    traversalPath: List[TraversalHop2] = ()
    retrieveClassificationData: bool = False


@dataclass
class TraversalHop2(TcBaseObj):
    """
    Represents traversal hop definition for "forward" or "reverse" traversal on either a relation property or a
    reference property.
    
    :var propertyName: Source object property name.
    :var propertyType: Type of property. Supported values are "relation" or "reference".
    :var destinationType: Type of destination object. All subtypes of POM_object are supported.
    :var direction: Direction of traversal. Supported values are "forward" or "reverse".
    :var destinationObjectCriteria: It represents the criteria expression. The destination objects that satisfy the
    criteria expression will be returned. By default, destination object criteria expression is empty. Supported values
    are
    1.    $CurrentUserSessionProject, which is a system-defined criteria expression for traversal to the  destination
    objects which are assigned to the current user session project. (optional)
    """
    propertyName: str = ''
    propertyType: str = ''
    destinationType: str = ''
    direction: str = ''
    destinationObjectCriteria: str = ''


@dataclass
class UpdateMemberInput(TcBaseObj):
    """
    Represents the definition data for members that is to be updated on an existing Logical Object Type.
    
    :var logicalObjectType: An object of logical object type ( subtype of "Fnd0LogicalObject" ) to which member
    definitions are being updated.
    :var membersToBeUpdated: A map (string, MemberPropertyDefinition2) of name of member and it's definition which is
    to be updated.
    Key for this map would be a existing "member name" and value a structure which contains the new definition for this
    member.
    """
    logicalObjectType: BusinessObject = None
    membersToBeUpdated: MemberDefinitionMap = None


@dataclass
class AddMembersPresentedPropsInput2(TcBaseObj):
    """
    Represents the definition data for member and presented properties to be added to an existing Logical Object Type.
    
    :var logicalObjectType: An object of logical object type ( subtype of "Fnd0LogicalObject" ) to which members and
    presented properties are to be added.
    :var memberPropertiesDefinitions: A list of member defintions.
    :var presentedPropertiesDefinitions: A list of presented property defintions.
    """
    logicalObjectType: BusinessObject = None
    memberPropertiesDefinitions: List[MemberPropertyDefinition2] = ()
    presentedPropertiesDefinitions: List[PresentedPropertyDefinition] = ()


"""
A map of member and it's member definition information.
"""
MemberDefinitionMap = Dict[str, MemberPropertyDefinition2]
